@@ -50,18 +50,17 @@
 **
 */
 
-#include "HTUtils.h"
-#include <string.h>		/* strchr() */
-#include "HTString.h"
-#include "HTParse.h"		/* URL parsing function		*/
-#include "HTList.h"		/* HTList object		*/
-#include "HTAlert.h"		/* HTConfirm(), HTPrompt()	*/
-#include "HTAAUtil.h"		/* AA common to both sides	*/
-#include "HTAssoc.h"		/* Assoc list			*/
-#include "HTAABrow.h"		/* Implemented here		*/
-#include "HTUU.h"		/* Uuencoding and uudecoding	*/
+#include <HTUtils.h>
+#include <HTString.h>
+#include <HTParse.h>		/* URL parsing function		*/
+#include <HTList.h>		/* HTList object		*/
+#include <HTAlert.h>		/* HTConfirm(), HTPrompt()	*/
+#include <HTAAUtil.h>		/* AA common to both sides	*/
+#include <HTAssoc.h>		/* Assoc list			*/
+#include <HTAABrow.h>		/* Implemented here		*/
+#include <HTUU.h>		/* Uuencoding and uudecoding	*/
 
-#include "LYLeaks.h"
+#include <LYLeaks.h>
 
 extern BOOL using_proxy;	/* Are we using an HTTP gateway? */
 
@@ -341,32 +340,29 @@ PRIVATE HTAASetup *HTAASetup_lookup ARGS4(
 
 	HTList *cur = server->setups;
 
-	if (TRACE)
-	    fprintf(stderr, "%s %s (%s:%d:%s)\n",
-			    "HTAASetup_lookup: resolving setup for",
-			    (IsProxy ? "proxy" : "server"),
-			    hostname, portnumber, docname);
+	CTRACE(tfp, "%s %s (%s:%d:%s)\n",
+		    "HTAASetup_lookup: resolving setup for",
+		    (IsProxy ? "proxy" : "server"),
+		    hostname, portnumber, docname);
 
 	while (NULL != (setup = (HTAASetup*)HTList_nextObject(cur))) {
 	    if (HTAA_templateMatch(setup->template, docname)) {
-		if (TRACE)
-		    fprintf(stderr, "%s `%s' %s `%s'\n",
-				    "HTAASetup_lookup:", docname,
-				    "matched template", setup->template);
+		CTRACE(tfp, "%s `%s' %s `%s'\n",
+			    "HTAASetup_lookup:", docname,
+			    "matched template", setup->template);
 		return setup;
+	    } else {
+	        CTRACE(tfp, "%s `%s' %s `%s'\n",
+			    "HTAASetup_lookup:", docname,
+			    "did NOT match template", setup->template);
 	    }
-	    else if (TRACE)
-	        fprintf(stderr, "%s `%s' %s `%s'\n",
-				"HTAASetup_lookup:", docname,
-				"did NOT match template", setup->template);
 	} /* while setups remain */
     } /* if valid parameters and server found */
 
-    if (TRACE)
-        fprintf(stderr, "%s `%s' %s\n",
-		        "HTAASetup_lookup: No template matched",
-		        (docname ? docname : "(null)"),
-		        "(so probably not protected)");
+    CTRACE(tfp, "%s `%s' %s\n",
+		 "HTAASetup_lookup: No template matched",
+		 (docname ? docname : "(null)"),
+		 "(so probably not protected)");
 
     return NULL;	/* NULL in parameters, or not found */
 }
@@ -603,12 +599,11 @@ PRIVATE char *compose_auth_string ARGS3(
     realm = HTAARealm_lookup(setup->server->realms, realmname);
     if (!(realm &&
     	  realm->username && *realm->username &&
-	  realm->password && *realm->password) || setup->retry) {
+	  realm->password) || setup->retry) { 
 	if (!realm) {
-	    if (TRACE)
-	        fprintf(stderr, "%s `%s' %s\n",
-			        "compose_auth_string: realm:", realmname,
-			        "not found -- creating");
+	    CTRACE(tfp, "%s `%s' %s\n",
+			"compose_auth_string: realm:", realmname,
+			"not found -- creating");
 	    realm = HTAARealm_new(setup->server->realms,
 				  realmname, NULL, NULL);
 	}
@@ -675,7 +670,7 @@ PRIVATE char *compose_auth_string ARGS3(
 	     *  Signals to retry. - FM
 	     */
 	    return NULL;
-	} else if (*realm->username == '\0' || *realm->password == '\0') {
+	} else if (*realm->username == '\0') { 
 	    /*
 	     *  Signals to abort. - FM
 	     */
@@ -860,9 +855,8 @@ PUBLIC char *HTAA_composeAuth ARGS4(
     **  on server-side.  Life is hard.)
     */
     if (HTAAForwardAuth) {
-	if (TRACE)
-	    fprintf(stderr, "HTAA_composeAuth: %s\n",
-			    "Forwarding received authorization");
+	CTRACE(tfp, "HTAA_composeAuth: %s\n",
+		    "Forwarding received authorization");
 	StrAllocCopy(HTAA_composeAuthResult, HTAAForwardAuth);
 	HTAAForwardAuth_reset();	/* Just a precaution */
 	return HTAA_composeAuthResult;
@@ -875,9 +869,7 @@ PUBLIC char *HTAA_composeAuth ARGS4(
 	**  Proxy Authorization required. - AJL
 	*/
 
-	if (TRACE)
-	    fprintf(stderr, 
-		    "Composing Proxy Authorization for %s:%d/%s\n",
+	CTRACE(tfp, "Composing Proxy Authorization for %s:%d/%s\n",
 		    hostname, portnumber, docname);
 
 	if (proxy_portnumber != portnumber ||
@@ -952,9 +944,7 @@ PUBLIC char *HTAA_composeAuth ARGS4(
 	/*
 	**  Normal WWW authorization.
 	*/
-	if (TRACE)
-	    fprintf(stderr, 
-		    "Composing Authorization for %s:%d/%s\n",
+	CTRACE(tfp, "Composing Authorization for %s:%d/%s\n",
 		    hostname, portnumber, docname);
 
 	if (current_portnumber != portnumber ||
@@ -1061,10 +1051,9 @@ PUBLIC char *HTAA_composeAuth ARGS4(
 **				  field (in function HTAA_composeAuth()).
 **			NO, otherwise.
 */
-PUBLIC BOOL HTAA_shouldRetryWithAuth ARGS5(
+PUBLIC BOOL HTAA_shouldRetryWithAuth ARGS4(
 	char *,		start_of_headers,
 	int,		length,
-	void *,		handle,
 	int,		soc,
 	BOOL,		IsProxy)
 {
@@ -1087,13 +1076,11 @@ PUBLIC BOOL HTAA_shouldRetryWithAuth ARGS5(
     /*
     **  Read server reply header lines
     */
-    if (TRACE)
-	fprintf(stderr, "Server reply header lines:\n");
+    CTRACE(tfp, "Server reply header lines:\n");
 
-    HTAA_setupReader(start_of_headers, length, handle, soc);
+    HTAA_setupReader(start_of_headers, length, soc);
     while (NULL != (line = HTAA_getUnfoldedLine())  &&  *line != '\0') {
-	if (TRACE)
-	    fprintf(stderr, "%s\n", line);
+	CTRACE(tfp, "%s\n", line);
 
 	if (strchr(line, ':')) {	/* Valid header line */
 
@@ -1133,9 +1120,8 @@ PUBLIC BOOL HTAA_shouldRetryWithAuth ARGS5(
 		    }
 		    scheme_specifics[scheme] = HTAA_parseArgList(args);
 		    num_schemes++;
-		}
-		else if (TRACE) {
-		    fprintf(stderr, "Unknown scheme `%s' %s\n",
+		} else {
+		    CTRACE(tfp, "Unknown scheme `%s' %s\n",
 			    (arg1 ? arg1 : "(null)"),
 			    (IsProxy ?
 			     "in Proxy-Authenticate: field" :
@@ -1145,14 +1131,13 @@ PUBLIC BOOL HTAA_shouldRetryWithAuth ARGS5(
 
 	    else if (!IsProxy &&
 		     0==strcasecomp(fieldname, "WWW-Protection-Template:")) {
-		if (TRACE)
-		    fprintf(stderr, "Protection template set to `%s'\n", arg1);
+		CTRACE(tfp, "Protection template set to `%s'\n", arg1);
 		StrAllocCopy(template, arg1);
 	    }
 
 	} /* if a valid header line */
-	else if (TRACE) {
-	    fprintf(stderr, "Invalid header line `%s' ignored\n", line);
+	else {
+	    CTRACE(tfp, "Invalid header line `%s' ignored\n", line);
 	} /* else invalid header line */
 
 	FREE(line);
