@@ -25,25 +25,21 @@
 **		     relayed from HTLoadGopher. - FM
 */
 
-#include "HTUtils.h"
-#include "tcp.h"
-#include "HTAlert.h"
-#include "HTML.h"
-#include "HTParse.h"
-#include "HTFormat.h"
-#include "HTTCP.h"
-#include "HTString.h"
-#include "HTFinger.h"
+#include <HTUtils.h>
+#include <HTAlert.h>
+#include <HTML.h>
+#include <HTParse.h>
+#include <HTFormat.h>
+#include <HTTCP.h>
+#include <HTString.h>
+#include <HTFinger.h>
 
-#include "LYLeaks.h"
-
-/* #define TRACE 1 */
+#include <LYUtils.h>
+#include <LYLeaks.h>
 
 #define FINGER_PORT 79		/* See rfc742 */
 #define BIG 1024		/* Bug */
 
-#define FREE(x) if (x) {free(x); x = NULL;}
-
 #define PUTC(c) (*targetClass.put_character)(target, c)
 #define PUTS(s) (*targetClass.put_string)(target, s)
 #define START(e) (*targetClass.start_element)(target, e, 0, 0, -1, 0)
@@ -129,13 +125,10 @@ PRIVATE int response ARGS5(
 
     /* Send the command.
     */
-    if (TRACE) 
-        fprintf(stderr, "HTFinger command to be sent: %s", command);
+    CTRACE(tfp, "HTFinger command to be sent: %s", command);
     status = NETWRITE(s, (char *)command, length);
     if (status < 0) {
-        if (TRACE)
-	    fprintf(stderr,
-                    "HTFinger: Unable to send command. Disconnecting.\n");
+	CTRACE(tfp, "HTFinger: Unable to send command. Disconnecting.\n");
         NETCLOSE(s);
         s = -1;
         return status;
@@ -148,8 +141,7 @@ PRIVATE int response ARGS5(
 
     /* Create the results report.
     */
-    if (TRACE)
-	fprintf(stderr,"HTFinger: Reading finger information\n");
+    CTRACE(tfp,"HTFinger: Reading finger information\n");
     START(HTML_HTML);
     PUTS("\n");
     START(HTML_HEAD);
@@ -190,10 +182,7 @@ PRIVATE int response ARGS5(
     while ((ch=NEXT_CHAR) != (char)EOF) {
 
 	if (interrupted_in_htgetcharacter) {
-	    if (TRACE) {
-	        fprintf(stderr,
-		  "HTFinger: Interrupted in HTGetCharacter, apparently.\n");
-	    }
+	    CTRACE(tfp, "HTFinger: Interrupted in HTGetCharacter, apparently.\n");
 	    _HTProgress ("Connection interrupted.");
 	    goto end_html;
         }
@@ -270,9 +259,7 @@ PUBLIC int HTLoadFinger ARGS4(
     int port;				/* Port number from URL */
     int status;				/* tcp return */
   
-    if (TRACE) {
-        fprintf(stderr, "HTFinger: Looking for %s\n", (arg ? arg : "NULL"));
-    }
+    CTRACE(tfp, "HTFinger: Looking for %s\n", (arg ? arg : "NULL"));
   
     if (!(arg && *arg)) {
         HTAlert("Could not load data.");
@@ -391,17 +378,13 @@ PUBLIC int HTLoadFinger ARGS4(
     /* Now, let's get a stream setup up from the FingerHost:
     ** CONNECTING to finger host
     */
-    if (TRACE)
-        fprintf(stderr, "HTFinger: doing HTDoConnect on '%s'\n", str);
+    CTRACE(tfp, "HTFinger: doing HTDoConnect on '%s'\n", str);
     status = HTDoConnect(str, "finger", FINGER_PORT, &s);
-    if (TRACE)
-        fprintf(stderr, "HTFinger: Done DoConnect; status %d\n", status);
+    CTRACE(tfp, "HTFinger: Done DoConnect; status %d\n", status);
 
     if (status == HT_INTERRUPTED) {
         /* Interrupt cleanly */
-	if (TRACE)
-	    fprintf(stderr,
-	    	  "HTFinger: Interrupted on connect; recovering cleanly.\n");
+	CTRACE(tfp, "HTFinger: Interrupted on connect; recovering cleanly.\n");
 	HTProgress ("Connection interrupted.");
 	FREE(str);
 	FREE(command);
@@ -410,15 +393,13 @@ PUBLIC int HTLoadFinger ARGS4(
     if (status < 0) {
         NETCLOSE(s);
 	s = -1;
-	if (TRACE) 
-	    fprintf(stderr, "HTFinger: Unable to connect to finger host.\n");
+	CTRACE(tfp, "HTFinger: Unable to connect to finger host.\n");
         HTAlert("Could not access finger host.");
 	FREE(str);
 	FREE(command);
 	return HT_NOT_LOADED;	/* FAIL */
     }
-    if (TRACE)
-        fprintf(stderr, "HTFinger: Connected to finger host '%s'.\n", str);
+    CTRACE(tfp, "HTFinger: Connected to finger host '%s'.\n", str);
     FREE(str);
 
     /* Send the command, and process response if successful.
