@@ -6,15 +6,10 @@
 **	New nodes are inserted between the header and the rest of the list.
 */
 
-#include "HTUtils.h"
-#include "HTList.h"
-
-/*#include <stdio.h> included by HTUtils.h -- FM *//* joe@athena, TBL 921019 */
-
-#include "LYLeaks.h"
-
-#define FREE(x) if (x) {free(x); x = NULL;}
+#include <HTUtils.h>
+#include <HTList.h>
 
+#include <LYLeaks.h>
 
 /*	Create list.
 */
@@ -63,10 +58,9 @@ PUBLIC void HTList_addObject ARGS2(
 	newNode->next = me->next;
 	me->next = newNode;
 
-    } else if (TRACE) {
-        fprintf(stderr,
-		"HTList: Trying to add object %p to a nonexisting list\n",
-		newObject);
+    } else {
+        CTRACE(tfp, "HTList: Trying to add object %p to a nonexisting list\n",
+		    newObject);
     }
 
     return;
@@ -106,20 +100,14 @@ PUBLIC void HTList_insertObjectAt ARGS3(
     int Pos = pos;
 
     if (!temp) {
-	if (TRACE) {
-	    fprintf(stderr,
-		    "HTList: Trying to add object %p to a nonexisting list\n",
+	CTRACE(tfp, "HTList: Trying to add object %p to a nonexisting list\n",
 		    newObject);
-	}
 	return;
     }
     if (Pos < 0) {
 	Pos = 0;
-	if (TRACE) {
-	    fprintf(stderr,
-		    "HTList: Treating negative object position %d as %d.\n",
+	CTRACE(tfp, "HTList: Treating negative object position %d as %d.\n",
 		    pos, Pos);
-	}
     }
 
     prevNode = temp;
