@@ -11,9 +11,7 @@
 #ifndef HTUU_H
 #define HTUU_H
 
-#ifndef HTUTILS_H
-#include "HTUtils.h"
-#endif /* HTUTILS_H */
+#include <HTUtils.h>
 
 PUBLIC int HTUU_encode PARAMS((unsigned char *bufin,
                                unsigned int nbytes,
@@ -24,6 +22,3 @@ PUBLIC int HTUU_decode PARAMS((char *bufcoded,
                                int outbufsize));
 
 #endif
-/*
-
-   End of file.  */
