@@ -8,12 +8,12 @@
 #define HTML_H
 
 #ifndef HTUTILS_H
-#include "HTUtils.h"
+#include <HTUtils.h>
 #endif /* HTUTILS_H */
-#include "UCDefs.h"
-#include "UCAux.h"
-#include "HTAnchor.h"
-#include "HTMLDTD.h"
+#include <UCDefs.h>
+#include <UCAux.h>
+#include <HTAnchor.h>
+#include <HTMLDTD.h>
 
 #ifdef SHORT_NAMES
 #define HTMLPresentation        HTMLPren
@@ -207,9 +207,7 @@ struct _HTStructured {
     int 		tag_charset; /* charset for attribute values etc. */
 };
 
-extern  HTStyle *styles[HTML_ELEMENTS+31]; /* adding 24 nested list styles  */
-					   /* and 3 header alignment styles */
-					   /* and 3 div alignment styles    */
+extern  HTStyle *LYstyles PARAMS((int style_number));
 
 /*
  *	Semi-Private functions. - FM
