@@ -1,6 +1,10 @@
 #ifndef HTSAVETOFILE_H
 #define HTSAVETOFILE_H
 
+#include <HTUtils.h>
+#include <HTStream.h>
+#include <HTFormat.h>
+
 extern HTStream * HTSaveToFile PARAMS((
         HTPresentation *        pres,
         HTParentAnchor *        anchor,
