@@ -17,12 +17,10 @@
 **		- FM					1997-08-02
 **
 **  TO DO: (roughly in order of decreasing priority)
-      * A means to specify "always allow" and "never allow" domains via
-	a configuration file is needed.
       * Hex escaping isn't considered at all.  Any semi-colons, commas,
 	or spaces actually in cookie names or values (i.e., not serving
 	as punctuation for the overall Set-Cookie value) should be hex
-	escaped if not quoted, but presumeably the server is expecting
+	escaped if not quoted, but presumably the server is expecting
 	them to be hex escaped in our Cookie request header as well, so
 	in theory we need not unescape them.  We'll see how this works
 	out in practice.
@@ -40,32 +38,31 @@
 	of use.
       * If a cookie has the secure flag set, we presently treat only SSL
 	connections as secure.	This may need to be expanded for other
-	secure communication protocols that become standarized.
-      * Cookies could be optionally stored in a file from session to session.
+	secure communication protocols that become standardized.
 */
 
-#include "HTUtils.h"
-#include "tcp.h"
-#include "HTAccess.h"
-#include "HTParse.h"
-#include "HTAlert.h"
-#include "LYCurses.h"
-#include "LYSignal.h"
-#include "LYUtils.h"
-#include "LYCharUtils.h"
-#include "LYClean.h"
-#include "LYGlobalDefs.h"
-#include "LYEdit.h"
-#include "LYStrings.h"
-#include "LYSystem.h"
-#include "GridText.h"
-#include "LYUtils.h"
-#include "LYCharUtils.h"
-#include "LYCookie.h"
+#include <HTUtils.h>
+#include <HTAccess.h>
+#include <HTParse.h>
+#include <HTAlert.h>
+#include <LYCurses.h>
+#include <LYSignal.h>
+#include <LYUtils.h>
+#include <LYCharUtils.h>
+#include <LYClean.h>
+#include <LYGlobalDefs.h>
+#include <LYEdit.h>
+#include <LYStrings.h>
+#include <GridText.h>
+#include <LYUtils.h>
+#include <LYCharUtils.h>
+#include <LYCookie.h>
 
-#include "LYLeaks.h"
+#include <LYLeaks.h>
 
-#define FREE(x) if (x) {free(x); x = NULL;}
+#define max_cookies_domain 50
+#define max_cookies_global 500
+#define max_cookies_buffer 4096
 
 /*
 **  The first level of the cookie list is a list indexed by the domain
@@ -73,7 +70,7 @@
 **  list.  Thus, finding the cookies that apply to a given URL is a
 **  two-level scan; first we check each domain to see if it applies,
 **  and if so, then we check the paths of all the cookies on that
-**  list.   We keep a running total of cookies as we add or delete
+**  list.  We keep a running total of cookies as we add or delete
 **  them
 */
 PRIVATE HTList *domain_list = NULL;
@@ -95,6 +92,7 @@ struct _cookie {
     int flags;	   /* Various flags */
     time_t expires;/* The time when this cookie expires */
     BOOL quoted;   /* Was a value quoted in the Set-Cookie header? */
+    BOOL from_file; /* Was this cookie loaded from the file jar? - RP */
 };
 typedef struct _cookie cookie;
 
@@ -103,6 +101,8 @@ typedef struct _cookie cookie;
 #define COOKIE_FLAG_EXPIRES_SET 4  /* If set, an expiry date was set */
 #define COOKIE_FLAG_DOMAIN_SET 8   /* If set, an non-default domain was set */
 #define COOKIE_FLAG_PATH_SET 16    /* If set, an non-default path was set */
+#define COOKIE_FLAG_PERSISTENT 32  /* If set, this cookie was persistent */
+
 struct _HTStream
 {
   HTStreamClass * isa;
@@ -285,9 +285,7 @@ PRIVATE void store_cookie ARGS3(
      *	not a prefix of the request-URI.
      */
     if (strncmp(co->path, path, co->pathlen) != 0) {
-	if (TRACE)
-	    fprintf(stderr,
-	    "store_cookie: Rejecting because '%s' is not a prefix of '%s'.\n",
+	CTRACE(tfp, "store_cookie: Rejecting because '%s' is not a prefix of '%s'.\n",
 		    co->path, path);
 	freeCookie(co);
 	co = NULL;
@@ -302,9 +300,7 @@ PRIVATE void store_cookie ARGS3(
 	 *  The hostname does not contain a dot.
 	 */
 	if (strchr(hostname, '.') == NULL) {
-	    if (TRACE)
-		fprintf(stderr,
-			"store_cookie: Rejecting because '%s' has no dot.\n",
+	    CTRACE(tfp, "store_cookie: Rejecting because '%s' has no dot.\n",
 			hostname);
 	    freeCookie(co);
 	    co = NULL;
@@ -319,9 +315,7 @@ PRIVATE void store_cookie ARGS3(
 	 *  value otherwise qualified. - FM
 	 */
 	if (co->domain[0] != '.' || co->domain[1] == '\0') {
-	    if (TRACE)
-		fprintf(stderr,
-			"store_cookie: Rejecting domain '%s'.\n",
+	    CTRACE(tfp, "store_cookie: Rejecting domain '%s'.\n",
 			co->domain);
 	    freeCookie(co);
 	    co = NULL;
@@ -329,9 +323,7 @@ PRIVATE void store_cookie ARGS3(
 	}
 	ptr = strchr((co->domain + 1), '.');
 	if (ptr == NULL || ptr[1] == '\0') {
-	    if (TRACE)
-		fprintf(stderr,
-			"store_cookie: Rejecting domain '%s'.\n",
+	    CTRACE(tfp, "store_cookie: Rejecting domain '%s'.\n",
 			co->domain);
 	    freeCookie(co);
 	    co = NULL;
@@ -343,9 +335,7 @@ PRIVATE void store_cookie ARGS3(
 	 *  not domain-match the Domain attribute.
 	 */
 	if (!host_matches(hostname, co->domain)) {
-	    if (TRACE)
-		fprintf(stderr,
-			"store_cookie: Rejecting domain '%s' for host '%s'.\n",
+	    CTRACE(tfp, "store_cookie: Rejecting domain '%s' for host '%s'.\n",
 			co->domain, hostname);
 	    freeCookie(co);
 	    co = NULL;
@@ -365,21 +355,22 @@ PRIVATE void store_cookie ARGS3(
 				strlen(INVALID_COOKIE_DOMAIN_CONFIRMATION) +
 				1));
 
-	    sprintf(msg,
-		    INVALID_COOKIE_DOMAIN_CONFIRMATION,
-		    co->domain,
-		    hostname);
-	    if (!HTConfirm(msg)) {
-		if (TRACE) {
-		    fprintf(stderr,
-		       "store_cookie: Rejecting domain '%s' for host '%s'.\n",
-			    co->domain,
-			    hostname);
+	    if (msg == 0)
+		outofmem(__FILE__, "store_cookie");
+	    if (!LYAcceptAllCookies) {
+		sprintf(msg,
+			INVALID_COOKIE_DOMAIN_CONFIRMATION,
+			co->domain,
+			hostname);
+		if (!HTConfirm(msg)) {
+		    CTRACE(tfp, "store_cookie: Rejecting domain '%s' for host '%s'.\n",
+				co->domain,
+				hostname);
+		    freeCookie(co);
+		    co = NULL;
+		    FREE(msg);
+		    return;
 		}
-		freeCookie(co);
-		co = NULL;
-		FREE(msg);
-		return;
 	    }
 	    FREE(msg);
 	}
@@ -418,7 +409,22 @@ PRIVATE void store_cookie ARGS3(
 	    de = (domain_entry *)calloc(1, sizeof(domain_entry));
 	    if (de == NULL)
 		outofmem(__FILE__, "store_cookie");
-	    de->bv = QUERY_USER;
+#ifdef EXP_PERSISTENT_COOKIES
+	    /*
+	     * Ok, this is a problem.  The first cookie for a domain
+	     * effectively sets the policy for that whole domain - for
+	     * something like Netlink, where there are lots of websites
+	     * under www.netlink.co.uk, this isn't sensible.  However,
+	     * taking this sort of decision down to cookie level also
+	     * isn't sensible.	Perhaps something based on the domain
+	     * and the path in conjunction makes more sense?  - RP
+	     */
+	    if (persistent_cookies
+	     && (co->flags & COOKIE_FLAG_PERSISTENT))
+		de->bv = FROM_FILE;
+	    else
+#endif
+		de->bv = QUERY_USER;
 	    cookie_list = de->cookie_list = HTList_new();
 	    StrAllocCopy(de->domain, co->domain);
 	    HTList_addObject(domain_list, de);
@@ -473,20 +479,16 @@ PRIVATE void store_cookie ARGS3(
     /*
      *	Don't add the cookie if we're over the domain's limit. - FM
      */
-    } else if (HTList_count(cookie_list) > 50) {
-	if (TRACE)
-	    fprintf(stderr,
-	"store_cookie: Domain's cookie limit exceeded!  Rejecting cookie.\n");
+    } else if (HTList_count(cookie_list) > max_cookies_domain) {
+	CTRACE(tfp, "store_cookie: Domain's cookie limit exceeded!  Rejecting cookie.\n");
 	freeCookie(co);
 	co = NULL;
 
     /*
      *	Don't add the cookie if we're over the total cookie limit. - FM
      */
-    } else if (total_cookies > 500) {
-	if (TRACE)
-	    fprintf(stderr,
-	"store_cookie: Total cookie limit exceeded!  Rejecting cookie.\n");
+    } else if (total_cookies > max_cookies_global) {
+	CTRACE(tfp, "store_cookie: Total cookie limit exceeded!  Rejecting cookie.\n");
 	freeCookie(co);
 	co = NULL;
 
@@ -503,9 +505,25 @@ PRIVATE void store_cookie ARGS3(
      *	Get confirmation if we need it, and add cookie
      *	if confirmed or 'allow' is set to always. - FM
      */
-    } else if (HTConfirmCookie(de, hostname,
-			       co->domain, co->path, co->name, co->value)) {
+    } else if (HTConfirmCookie(de, hostname, co->name, co->value)) {
+
+#ifdef EXP_PERSISTENT_COOKIES
+	/*
+	 * If the cookie domain came from persistent cookie file,
+	 * we want to add new cookies to the end of the cookie list
+	 * to maintain their order for servers that need cookies in
+	 * a particular order. This is a hack.
+	 */
+	if (persistent_cookies) {
+	    if ((de->bv = FROM_FILE) != 0) {
+		HTList_appendObject(cookie_list, co);
+	    } else {
+		HTList_insertObjectAt(cookie_list, co, pos);
+	    }
+	}
+#else
 	HTList_insertObjectAt(cookie_list, co, pos);
+#endif /* EXP_PERSISTENT_COOKIES */
 	total_cookies++;
     } else {
 	freeCookie(co);
@@ -536,12 +554,12 @@ PRIVATE char * scan_cookie_sublist ARGS6(
 	co = (cookie *)hl->object;
 	next = hl->next;
 
-	if (TRACE && co) {
-	    fprintf(stderr, "Checking cookie %lx %s=%s\n",
+	if (co) {
+	    CTRACE(tfp, "Checking cookie %lx %s=%s\n",
 			    (long)hl,
 			    (co->name ? co->name : "(no name)"),
 			    (co->value ? co->value : "(no value)"));
-	    fprintf(stderr, "%s %s %d %s %s %d%s\n",
+	    CTRACE(tfp, "%s %s %d %s %s %d%s\n",
 			    hostname,
 			    (co->domain ? co->domain : "(no domain)"),
 			    host_matches(hostname, co->domain),
@@ -611,10 +629,27 @@ PRIVATE char * scan_cookie_sublist ARGS6(
 		/*
 		 *  Check if we should fold the header. - FM
 		 */
-		if (len > 800) {
-		    StrAllocCat(header, crlftab);
-		    len = 0;
-		}
+
+		/*
+		 * Section 2.2 of RFC1945 says:
+		 *
+		 *  HTTP/1.0 headers may be folded onto multiple lines
+		 *  if each continuation line begins with a space or
+		 *  horizontal tab. All linear whitespace, including
+		 *  folding, has the same semantics as SP.
+		 *  [...]
+		 *  However, folding of header lines is not expected by
+		 *  some applications, and should not be generated by
+		 *  HTTP/1.0 applications.
+		 *
+		 * This code was causing problems.  Let's not use it. -BJP
+		 */
+
+		/* if (len > 800) { */
+		/*    StrAllocCat(header, crlftab); */
+		/*    len = 0; */
+		/* } */
+
 	    }
 	    /*
 	     *	Include the cookie name=value pair.
@@ -718,14 +753,12 @@ PRIVATE void LYProcessSetCookies ARGS6(
      *	adding each cookie to the CombinedCookies list. - FM
      */
     p = (SetCookie2 ? SetCookie2 : "");
-    if (TRACE && SetCookie && *p) {
-	fprintf(stderr, "LYProcessSetCookies: Using Set-Cookie2 header.\n");
+    if (SetCookie && *p) {
+	CTRACE(tfp, "LYProcessSetCookies: Using Set-Cookie2 header.\n");
     }
-    while (NumCookies <= 50 && *p) {
+    while (NumCookies <= max_cookies_domain && *p) {
 	attr_start = attr_end = value_start = value_end = NULL;
-	while (*p != '\0' && isspace((unsigned char)*p)) {
-	    p++;
-	}
+	p = LYSkipCBlanks(p);
 	/*
 	 *  Get the attribute name.
 	 */
@@ -734,9 +767,7 @@ PRIVATE void LYProcessSetCookies ARGS6(
 	       *p != '=' && *p != ';' && *p != ',')
 	    p++;
 	attr_end = p;
-	while (*p != '\0' && isspace((unsigned char)*p)) {
-	    p++;
-	}
+	p = LYSkipCBlanks(p);
 
 	/*
 	 *  Check for an '=' delimiter, or an 'expires' name followed
@@ -756,9 +787,7 @@ PRIVATE void LYProcessSetCookies ARGS6(
 	    if (*p == '=') {
 		p++;
 	    }
-	    while (*p != '\0' && isspace((unsigned char)*p)) {
-		p++;
-	    }
+	    p = LYSkipCBlanks(p);
 	    /*
 	     *	Hack alert!  We must handle Netscape-style cookies with
 	     *		"Expires=Mon, 01-Jan-96 13:45:35 GMT" or
@@ -820,20 +849,16 @@ PRIVATE void LYProcessSetCookies ARGS6(
 		    while (isdigit((unsigned char)*p))
 			p++;
 		    value_end = p;
-		    while (isspace((unsigned char)*p))
-			p++;
+		    p = LYSkipCBlanks(p);
 		    if (*p == '\0' || *p == ';')
 			break;
 		    if (*p == ',') {
-			cp = (p + 1);
-			while (*cp != '\0' && isspace((unsigned char)*cp))
-			    cp++;
+			cp = LYSkipCBlanks(p + 1);
 			if (*cp != '\0' && isdigit((unsigned char)*cp)) {
 			    cp1 = cp;
 			    while (isdigit((unsigned char)*cp1))
 				cp1++;
-			    while (*cp != '\0' && isspace((unsigned char)*cp))
-				cp1++;
+			    cp1 = LYSkipCBlanks(cp1);
 			    if (*cp1 == '\0' || *cp1 == ',' || *cp1 == ';') {
 				p = cp;
 				continue;
@@ -908,8 +933,8 @@ PRIVATE void LYProcessSetCookies ARGS6(
 	    if (value_end > value_start) {
 		int value_len = (value_end - value_start);
 
-		if (value_len > 4096) {
-		    value_len = 4096;
+		if (value_len > max_cookies_buffer) {
+		    value_len = max_cookies_buffer;
 		}
 		value = (char *)calloc(1, value_len + 1);
 		if (value == NULL)
@@ -976,9 +1001,7 @@ PRIVATE void LYProcessSetCookies ARGS6(
 			 url_type == HTTPS_URL_TYPE)) {
 			length += strlen(cur_cookie->commentURL);
 		    } else {
-			if (TRACE)
-			    fprintf(stderr,
-		     "LYProcessSetCookies: Rejecting commentURL value '%s'\n",
+			CTRACE(tfp, "LYProcessSetCookies: Rejecting commentURL value '%s'\n",
 				    cur_cookie->commentURL);
 			FREE(cur_cookie->commentURL);
 		    }
@@ -1007,11 +1030,9 @@ PRIVATE void LYProcessSetCookies ARGS6(
 				   isdigit((unsigned char)*ptr))
 				ptr++;
 			    if (*ptr != '\0') {
-				if (TRACE) {
-				    fprintf(stderr,
+				CTRACE(tfp,
 	       "LYProcessSetCookies: Adding lead dot for domain value '%s'\n",
 					    value);
-				}
 				StrAllocCopy(cur_cookie->domain, ".");
 				StrAllocCat(cur_cookie->domain, value);
 			    } else {
@@ -1094,9 +1115,7 @@ PRIVATE void LYProcessSetCookies ARGS6(
 			cur_cookie->expires = (time_t)0;
 		    } else {
 			cur_cookie->expires = (time(NULL) + temp);
-			if (TRACE)
-			    fprintf(stderr,
-				    "LYSetCookie: expires %ld, %s",
+			CTRACE(tfp, "LYSetCookie: expires %ld, %s",
 				    (long) cur_cookie->expires,
 				    ctime(&cur_cookie->expires));
 		    }
@@ -1119,9 +1138,7 @@ PRIVATE void LYProcessSetCookies ARGS6(
 			cur_cookie->flags |= COOKIE_FLAG_EXPIRES_SET;
 			cur_cookie->expires = LYmktime(value, FALSE);
 			if (cur_cookie->expires > 0) {
-			    if (TRACE)
-				fprintf(stderr,
-					"LYSetCookie: expires %ld, %s",
+			    CTRACE(tfp, "LYSetCookie: expires %ld, %s",
 					(long) cur_cookie->expires,
 					ctime(&cur_cookie->expires));
 			}
@@ -1142,7 +1159,7 @@ PRIVATE void LYProcessSetCookies ARGS6(
 		 *  If we've started a cookie, and it's not too big,
 		 *  save it in the CombinedCookies list. - FM
 		 */
-		if (length <= 4096 && cur_cookie != NULL) {
+		if (length <= max_cookies_buffer && cur_cookie != NULL) {
 		    /*
 		     *	Assume version 1 if not set to that or higher. - FM
 		     */
@@ -1151,16 +1168,14 @@ PRIVATE void LYProcessSetCookies ARGS6(
 		    }
 		    HTList_appendObject(CombinedCookies, cur_cookie);
 		} else if (cur_cookie != NULL) {
-		    if (TRACE) {
-			fprintf(stderr,
+		    CTRACE(tfp,
 			"LYProcessSetCookies: Rejecting Set-Cookie2: %s=%s\n",
 				(cur_cookie->name ?
 				 cur_cookie->name : "[no name]"),
 				(cur_cookie->value ?
 				 cur_cookie->value : "[no value]"));
-			fprintf(stderr,
-			"                     due to excessive length!\n");
-		    }
+		    CTRACE(tfp,
+			   "                     due to excessive length!\n");
 		    freeCookie(cur_cookie);
 		    cur_cookie = NULL;
 		}
@@ -1190,23 +1205,24 @@ PRIVATE void LYProcessSetCookies ARGS6(
      *	Add any final SetCookie2 cookie to the CombinedCookie list
      *	if we are within the length limit. - FM
      */
-    if (NumCookies <= 50 && length <= 4096 && cur_cookie != NULL) {
+    if (NumCookies <= max_cookies_domain
+     && length <= max_cookies_buffer
+     && cur_cookie != NULL) {
 	if (cur_cookie->version < 1) {
 	    cur_cookie->version = 1;
 	}
 	HTList_appendObject(CombinedCookies, cur_cookie);
     } else if (cur_cookie != NULL) {
-	if (TRACE) {
-	    fprintf(stderr,
-	 "LYProcessSetCookies: Rejecting Set-Cookie2: %s=%s\n",
+	CTRACE(tfp, "LYProcessSetCookies: Rejecting Set-Cookie2: %s=%s\n",
 		    (cur_cookie->name ? cur_cookie->name : "[no name]"),
 		    (cur_cookie->value ? cur_cookie->value : "[no value]"));
-	    fprintf(stderr,
-	 "                     due to excessive %s%s%s\n",
-		    (length > 4096 ? "length" : ""),
-		    (length > 4096 && NumCookies > 50 ? " and " : ""),
-		    (NumCookies > 50 ? "number!\n" : "!\n"));
-	}
+	CTRACE(tfp, "                     due to excessive %s%s%s\n",
+		    (length > max_cookies_buffer ? "length" : ""),
+		    (length > max_cookies_buffer &&
+		     NumCookies > max_cookies_domain
+			? " and "
+			: ""),
+		    (NumCookies > max_cookies_domain ? "number!\n" : "!\n"));
 	freeCookie(cur_cookie);
 	cur_cookie = NULL;
     }
@@ -1219,14 +1235,12 @@ PRIVATE void LYProcessSetCookies ARGS6(
     NumCookies = 0;
     cur_cookie = NULL;
     p = ((SetCookie && !(SetCookie2 && *SetCookie2)) ? SetCookie : "");
-    if (TRACE && SetCookie2 && *p) {
-	fprintf(stderr, "LYProcessSetCookies: Using Set-Cookie header.\n");
+    if (SetCookie2 && *p) {
+	CTRACE(tfp, "LYProcessSetCookies: Using Set-Cookie header.\n");
     }
-    while (NumCookies <= 50 && *p) {
+    while (NumCookies <= max_cookies_domain && *p) {
 	attr_start = attr_end = value_start = value_end = NULL;
-	while (*p != '\0' && isspace((unsigned char)*p)) {
-	    p++;
-	}
+	p = LYSkipCBlanks(p);
 	/*
 	 *  Get the attribute name.
 	 */
@@ -1235,9 +1249,7 @@ PRIVATE void LYProcessSetCookies ARGS6(
 	       *p != '=' && *p != ';' && *p != ',')
 	    p++;
 	attr_end = p;
-	while (*p != '\0' && isspace((unsigned char)*p)) {
-	    p++;
-	}
+	p = LYSkipCBlanks(p);
 
 	/*
 	 *  Check for an '=' delimiter, or an 'expires' name followed
@@ -1253,9 +1265,7 @@ PRIVATE void LYProcessSetCookies ARGS6(
 	    if (*p == '=') {
 		p++;
 	    }
-	    while (*p != '\0' && isspace((unsigned char)*p)) {
-		p++;
-	    }
+	    p = LYSkipCBlanks(p);
 	    /*
 	     *	Hack alert!  We must handle Netscape-style cookies with
 	     *		"Expires=Mon, 01-Jan-96 13:45:35 GMT" or
@@ -1317,20 +1327,16 @@ PRIVATE void LYProcessSetCookies ARGS6(
 		    while (isdigit((unsigned char)*p))
 			p++;
 		    value_end = p;
-		    while (isspace((unsigned char)*p))
-			p++;
+		    p = LYSkipCBlanks(p);
 		    if (*p == '\0' || *p == ';')
 			break;
 		    if (*p == ',') {
-			cp = (p + 1);
-			while (*cp != '\0' && isspace((unsigned char)*cp))
-			    cp++;
+			cp = LYSkipCBlanks(p + 1);
 			if (*cp != '\0' && isdigit((unsigned char)*cp)) {
 			    cp1 = cp;
 			    while (isdigit((unsigned char)*cp1))
 				cp1++;
-			    while (*cp != '\0' && isspace((unsigned char)*cp))
-				cp1++;
+			    cp1 = LYSkipCBlanks(cp1);
 			    if (*cp1 == '\0' || *cp1 == ',' || *cp1 == ';') {
 				p = cp;
 				continue;
@@ -1406,8 +1412,8 @@ PRIVATE void LYProcessSetCookies ARGS6(
 	    if (value_end > value_start) {
 		int value_len = (value_end - value_start);
 
-		if (value_len > 4096) {
-		    value_len = 4096;
+		if (value_len > max_cookies_buffer) {
+		    value_len = max_cookies_buffer;
 		}
 		value = (char *)calloc(1, value_len + 1);
 		if (value == NULL)
@@ -1474,9 +1480,7 @@ PRIVATE void LYProcessSetCookies ARGS6(
 			 url_type == HTTPS_URL_TYPE)) {
 			length += strlen(cur_cookie->commentURL);
 		    } else {
-			if (TRACE)
-			    fprintf(stderr,
-		     "LYProcessSetCookies: Rejecting commentURL value '%s'\n",
+			CTRACE(tfp, "LYProcessSetCookies: Rejecting commentURL value '%s'\n",
 				    cur_cookie->commentURL);
 			FREE(cur_cookie->commentURL);
 		    }
@@ -1505,11 +1509,9 @@ PRIVATE void LYProcessSetCookies ARGS6(
 				   isdigit((unsigned char)*ptr))
 				ptr++;
 			    if (*ptr != '\0') {
-				if (TRACE) {
-				    fprintf(stderr,
+				CTRACE(tfp,
 	       "LYProcessSetCookies: Adding lead dot for domain value '%s'\n",
 					    value);
-				}
 				StrAllocCopy(cur_cookie->domain, ".");
 				StrAllocCat(cur_cookie->domain, value);
 			    } else {
@@ -1623,7 +1625,7 @@ PRIVATE void LYProcessSetCookies ARGS6(
 		 *  If we've started a cookie, and it's not too big,
 		 *  save it in the CombinedCookies list. - FM
 		 */
-		if (length <= 4096 && cur_cookie != NULL) {
+		if (length <= max_cookies_buffer && cur_cookie != NULL) {
 		    /*
 		     *	If we had a Set-Cookie2 header, make sure
 		     *	the version is at least 1, and mark it for
@@ -1637,16 +1639,12 @@ PRIVATE void LYProcessSetCookies ARGS6(
 		    }
 		    HTList_appendObject(CombinedCookies, cur_cookie);
 		} else if (cur_cookie != NULL) {
-		    if (TRACE) {
-			fprintf(stderr,
-			"LYProcessSetCookies: Rejecting Set-Cookie: %s=%s\n",
+		    CTRACE(tfp, "LYProcessSetCookies: Rejecting Set-Cookie: %s=%s\n",
 				(cur_cookie->name ?
 				 cur_cookie->name : "[no name]"),
 				(cur_cookie->value ?
 				 cur_cookie->value : "[no value]"));
-			fprintf(stderr,
-			"                     due to excessive length!\n");
-		    }
+		    CTRACE(tfp, "                     due to excessive length!\n");
 		    freeCookie(cur_cookie);
 		    cur_cookie = NULL;
 		}
@@ -1675,7 +1673,7 @@ PRIVATE void LYProcessSetCookies ARGS6(
     /*
      *	Handle the final Set-Cookie cookie if within length limit. - FM
      */
-    if (NumCookies <= 50 && length <= 4096 && cur_cookie != NULL) {
+    if (NumCookies <= max_cookies_domain && length <= max_cookies_buffer && cur_cookie != NULL) {
 	if (SetCookie2 != NULL) {
 	    if (cur_cookie->version < 1) {
 		cur_cookie->version = 1;
@@ -1684,17 +1682,13 @@ PRIVATE void LYProcessSetCookies ARGS6(
 	}
 	HTList_appendObject(CombinedCookies, cur_cookie);
     } else if (cur_cookie != NULL) {
-	if (TRACE) {
-	    fprintf(stderr,
-	  "LYProcessSetCookies: Rejecting Set-Cookie: %s=%s\n",
+	CTRACE(tfp, "LYProcessSetCookies: Rejecting Set-Cookie: %s=%s\n",
 		    (cur_cookie->name ? cur_cookie->name : "[no name]"),
 		    (cur_cookie->value ? cur_cookie->value : "[no value]"));
-	    fprintf(stderr,
-	  "                     due to excessive %s%s%s\n",
-		    (length > 4096 ? "length" : ""),
-		    (length > 4096 && NumCookies > 50 ? " and " : ""),
-		    (NumCookies > 50 ? "number!\n" : "!\n"));
-	}
+	CTRACE(tfp, "                     due to excessive %s%s%s\n",
+		    (length > max_cookies_buffer ? "length" : ""),
+		    (length > max_cookies_buffer && NumCookies > max_cookies_domain ? " and " : ""),
+		    (NumCookies > max_cookies_domain ? "number!\n" : "!\n"));
 	freeCookie(cur_cookie);
 	cur_cookie = NULL;
     }
@@ -1705,24 +1699,19 @@ PRIVATE void LYProcessSetCookies ARGS6(
      */
     cl = CombinedCookies;
     while (NULL != (co = (cookie *)HTList_nextObject(cl))) {
-	if (TRACE) {
-	    fprintf(stderr, "LYProcessSetCookie: attr=value pair: '%s=%s'\n",
+	CTRACE(tfp, "LYProcessSetCookie: attr=value pair: '%s=%s'\n",
 			    (co->name ? co->name : "[no name]"),
 			    (co->value ? co->value : "[no value]"));
-	    if (co->expires > 0) {
-		fprintf(stderr, "                    expires: %ld, %s\n",
-				 (long)co->expires,
-				 ctime(&co->expires));
-	    }
+	if (co->expires > 0) {
+		CTRACE(tfp, "                    expires: %ld, %s\n",
+			    (long)co->expires,
+			    ctime(&co->expires));
 	}
 	if (!strncasecomp(address, "https:", 6) &&
 	    LYForceSSLCookiesSecure == TRUE &&
 	    !(co->flags & COOKIE_FLAG_SECURE)) {
 	    co->flags |= COOKIE_FLAG_SECURE;
-	    if (TRACE) {
-		fprintf(stderr,
-			"                    Forced the 'secure' flag on.\n");
-	    }
+	    CTRACE(tfp, "                    Forced the 'secure' flag on.\n");
 	}
 	store_cookie(co, hostname, path);
     }
@@ -1780,23 +1769,19 @@ PUBLIC void LYSetCookie ARGS3(
 	 */
 	BadHeaders = TRUE;
     }
-    if (TRACE) {
-	fprintf(stderr,
-		"LYSetCookie called with host '%s', path '%s',\n",
+    CTRACE(tfp, "LYSetCookie called with host '%s', path '%s',\n",
 		(hostname ? hostname : ""),
 		(path ? path : ""));
-	if (SetCookie) {
-	    fprintf(stderr, "    and Set-Cookie: '%s'\n",
-			    (SetCookie ? SetCookie : ""));
-	}
-	if (SetCookie2) {
-	    fprintf(stderr, "    and Set-Cookie2: '%s'\n",
-			    (SetCookie2 ? SetCookie2 : ""));
-	}
-	if (LYSetCookies == FALSE || BadHeaders == TRUE) {
-	    fprintf(stderr,
-		    "    Ignoring this Set-Cookie/Set-Cookie2 request.\n");
-	}
+    if (SetCookie) {
+	CTRACE(tfp, "    and Set-Cookie: '%s'\n",
+			 (SetCookie ? SetCookie : ""));
+    }
+    if (SetCookie2) {
+	CTRACE(tfp, "    and Set-Cookie2: '%s'\n",
+			 (SetCookie2 ? SetCookie2 : ""));
+    }
+    if (LYSetCookies == FALSE || BadHeaders == TRUE) {
+	CTRACE(tfp, "    Ignoring this Set-Cookie/Set-Cookie2 request.\n");
     }
 
     /*
@@ -1831,13 +1816,10 @@ PUBLIC char * LYCookie ARGS4(
     HTList *hl = domain_list, *next = NULL;
     domain_entry *de;
 
-    if (TRACE) {
-	fprintf(stderr,
-		"LYCookie: Searching for '%s:%d', '%s'.\n",
+    CTRACE(tfp, "LYCookie: Searching for '%s:%d', '%s'.\n",
 		(hostname ? hostname : "(null)"),
 		port,
 		(path ? path : "(null)"));
-    }
 
     /*
      *	Search the cookie_list elements in the domain_list
@@ -1888,6 +1870,183 @@ PUBLIC char * LYCookie ARGS4(
     return(NULL);
 }
 
+#ifdef EXP_PERSISTENT_COOKIES
+/* rjp - experiment cookie loading */
+PUBLIC void LYLoadCookies ARGS1 (
+	CONST char *,	cookie_file)
+{
+    FILE *cookie_handle;
+    char buf[max_cookies_buffer+10]; /* should be long enough for a cookie line */
+    static char domain[256], path[LY_MAXPATH], name[256], value[4100];
+    static char what[8], secure[8], expires_a[16];
+    static struct {
+	char *s;
+	size_t n;
+    } tok_values[] = {
+	{ domain,	sizeof(domain) },
+	{ what,		sizeof(what) },
+	{ path,		sizeof(path) },
+	{ secure,	sizeof(secure) },
+	{ expires_a,	sizeof(expires_a) },
+	{ name,		sizeof(name) },
+	{ value,	sizeof(value) },
+	{ NULL, 0 }
+	};
+    time_t expires;
+
+    cookie_handle = fopen(cookie_file, "r+");
+    if (!cookie_handle)
+	return;
+
+    while (!feof(cookie_handle)) {
+	cookie *moo;
+	unsigned i = 0;
+	int tok_loop;
+	char *tok_out, *tok_ptr;
+	char *j;
+
+	j = fgets(buf, sizeof(buf)-1, cookie_handle);
+
+	if((j == NULL) || (buf[0] == '\0' || buf[0] == '\n')) {
+	    continue;
+	}
+
+
+	/*
+	 * Strip out the newline that fgets() puts at the end of a
+	 * cookie.
+	 */
+
+	while(buf[i] != '\n' && i < sizeof(buf)) {
+	    i++;
+	}
+	buf[i] = '\0';
+
+	/*
+	 * Tokenise the cookie line into its component parts -
+	 * this only works for Netscape style cookie files at the
+	 * moment.  It may be worth investigating an alternative
+	 * format for Lynx because the Netscape format isn't all
+	 * that useful, or future-proof. - RP
+	 *
+	 * 'fixed' by using strsep instead of strtok.  No idea
+	 * what kind of platform problems this might introduce. - RP
+	 */
+	tok_ptr = buf;
+	tok_out = LYstrsep(&tok_ptr, "\t");
+	for (tok_loop = 0; tok_out && tok_values[tok_loop].s; tok_loop++) {
+	CTRACE(tfp, ">%d:%p:%p:[%s]:%s\n",
+	    tok_loop, tok_values[tok_loop].s, tok_out, tok_out, buf);
+	    LYstrncpy(tok_values[tok_loop].s, tok_out, tok_values[tok_loop].n);
+	    /*
+	     * It looks like strtok ignores a leading delimiter,
+	     * which makes things a bit more interesting.  Something
+	     * like "FALSE\t\tFALSE\t" translates to FALSE,FALSE
+	     * instead of FALSE,,FALSE. - RP
+	     */
+	    tok_out = LYstrsep(&tok_ptr, "\t");
+	}
+	expires = atol(expires_a);
+	CTRACE(tfp, "COOKIE: expires %s\n", ctime(&expires));
+
+	/*
+	 * This fails when the path is blank
+	 *
+	 * sscanf(buf, "%s\t%s\t%s\t%s\t%d\t%s\t%[ -~]",
+	 *  domain, what, path, secure, &expires, name, value);
+	 */
+
+	CTRACE(tfp, "%s\t%s\t%s\t%s\t%ld\t%s\t%s\tREADCOOKIE\n",
+	    domain, what, path, secure, (long) expires, name, value);
+	moo = newCookie();
+	StrAllocCopy(moo->domain, domain);
+	StrAllocCopy(moo->path, path);
+	StrAllocCopy(moo->name, name);
+	StrAllocCopy(moo->value, value);
+	moo->pathlen = strlen(moo->path);
+	moo->flags |= COOKIE_FLAG_PERSISTENT;
+	moo->expires = expires;
+	/*
+	 * I don't like using this to store the cookies because it's
+	 * designed to store cookies that have been received from an
+	 * HTTP request, not from a persistent cookie jar.  Hence the
+	 * mucking about with the COOKIE_FLAG_PERSISTENT above. - RP
+	 */
+	store_cookie(moo, domain, path);
+    }
+    fclose (cookie_handle);
+}
+
+/* rjp - experimental persistent cookie support */
+PUBLIC void LYStoreCookies ARGS1 (
+	CONST char *,	cookie_file)
+{
+    char buf[1024];
+    HTList *dl, *cl;
+    domain_entry *de;
+    cookie *co;
+    FILE *cookie_handle;
+#ifdef VMS
+    extern BOOLEAN HadVMSInterrupt;
+#endif /* VMS */
+    time_t now = time(NULL); /* system specific? - RP */
+
+    /*
+     *	Check whether we have something to do. - FM
+     */
+    if (HTList_isEmpty(domain_list)) {
+	/* No cookies, so don't bother updating the file */
+	return;
+    }
+
+    cookie_handle = fopen(cookie_file, "w+");
+    for (dl = domain_list; dl != NULL; dl = dl->next) {
+	de = dl->object;
+	if (de == NULL)
+	    /*
+	     *	Fote says the first object is NULL.  Go with that.
+	     */
+	    continue;
+
+	switch (de->bv) {
+	case (ACCEPT_ALWAYS):
+	    sprintf(buf, COOKIES_ALWAYS_ALLOWED);
+	    break;
+	case (REJECT_ALWAYS):
+	    sprintf(buf, COOKIES_NEVER_ALLOWED);
+	    break;
+	case (QUERY_USER):
+	    sprintf(buf, COOKIES_ALLOWED_VIA_PROMPT);
+	    break;
+	case (FROM_FILE):
+	    sprintf(buf, "(From Cookie Jar)");
+	    break;
+	}
+
+	/*
+	 *  Show the domain's cookies. - FM
+	 */
+	for (cl = de->cookie_list; cl != NULL; cl = cl->next) {
+	    /*
+	     *	First object is always NULL. - FM
+	     */
+	    if ((co = (cookie *)cl->object) == NULL)
+		continue;
+
+	    CTRACE(tfp, "COOKIE: %ld cf %ld\n", (long) now, (long) co->expires);
+	    fprintf(cookie_handle, "%s\t%s\t%s\t%s\t%ld\t%s\t%s\n",
+		de->domain,
+		"FALSE", co->path,
+		co->flags & COOKIE_FLAG_SECURE ? "TRUE" : "FALSE",
+		(long) co->expires, co->name, co->value);
+	}
+    }
+    fclose(cookie_handle);
+
+    HTSYS_purge(cookie_file);
+}
+#endif
+
 /*	LYHandleCookies - F.Macrides (macrides@sci.wfeb.edu)
 **	---------------
 **
@@ -2044,9 +2203,8 @@ PRIVATE int LYHandleCookies ARGS4 (
 				 *  from this domain. - FM
 				 */
 				de->bv = QUERY_USER;
-				_user_message(ALWAYS_ALLOWING_COOKIES,
+				HTUserMsg2(ALWAYS_ALLOWING_COOKIES,
 					      de->domain);
-				sleep(MessageSecs);
 				return(HT_NO_DATA);
 
 			    case 'C':
@@ -2055,8 +2213,7 @@ PRIVATE int LYHandleCookies ARGS4 (
 				/*
 				 *  Cancelled. - FM
 				 */
-				_statusline(CANCELLED);
-				sleep(MessageSecs);
+				HTUserMsg(CANCELLED);
 				return(HT_NO_DATA);
 
 			    case 'D':
@@ -2117,13 +2274,12 @@ Delete_all_cookies_in_domain:
 
 			    case 'P':
 				/*
-				 *  Set to prompt for cookie acceptence
+				 *  Set to prompt for cookie acceptance
 				 *  from this domain. - FM
 				 */
 				de->bv = QUERY_USER;
-				_user_message(PROMTING_TO_ALLOW_COOKIES,
-					      de->domain);
-				sleep(MessageSecs);
+				HTUserMsg2(PROMPTING_TO_ALLOW_COOKIES,
+					   de->domain);
 				return(HT_NO_DATA);
 
 			    case 'V':
@@ -2132,9 +2288,8 @@ Delete_all_cookies_in_domain:
 				 *  from this domain. - FM
 				 */
 				de->bv = REJECT_ALWAYS;
-				_user_message(NEVER_ALLOWING_COOKIES,
-					      de->domain);
-				sleep(MessageSecs);
+				HTUserMsg2(NEVER_ALLOWING_COOKIES,
+					   de->domain);
 				if ((!HTList_isEmpty(de->cookie_list)) &&
 				    HTConfirm(DELETE_ALL_COOKIES_IN_DOMAIN))
 				    goto Delete_all_cookies_in_domain;
@@ -2186,10 +2341,9 @@ Delete_all_cookies_in_domain:
     sprintf(buf, "<HEAD>\n<TITLE>%s</title>\n</HEAD>\n<BODY>\n",
 		 COOKIE_JAR_TITLE);
     (*target->isa->put_block)(target, buf, strlen(buf));
-
-    sprintf(buf, "<H1>%s</H1>\n", REACHED_COOKIE_JAR_PAGE);
-    (*target->isa->put_block)(target, buf, strlen(buf));
-    sprintf(buf, "<H2>%s Version %s</H2>\n", LYNX_NAME, LYNX_VERSION);
+    sprintf(buf, "<h1>%s (%s), help on <a href=\"%s%s\">%s</a></h1>\n",
+	LYNX_NAME, LYNX_VERSION,
+	helpfilepath, COOKIE_JAR_HELP, COOKIE_JAR_TITLE);
     (*target->isa->put_block)(target, buf, strlen(buf));
 
     sprintf(buf, "<NOTE>%s\n", ACTIVATE_TO_GOBBLE);
@@ -2210,8 +2364,8 @@ Delete_all_cookies_in_domain:
 	/*
 	 *  Show the domain link and 'allow' setting. - FM
 	 */
-	sprintf(buf, "<DT><A HREF=\"LYNXCOOKIE://%s/\">Domain=%s</A>\n",
-		      de->domain, de->domain);
+	sprintf(buf, "<DT>%s<DD><A HREF=\"LYNXCOOKIE://%s/\">Domain=%s</A>\n",
+		      de->domain, de->domain, de->domain);
 	(*target->isa->put_block)(target, buf, strlen(buf));
 	switch (de->bv) {
 	    case (ACCEPT_ALWAYS):
@@ -2222,9 +2376,14 @@ Delete_all_cookies_in_domain:
 		break;
 	    case (QUERY_USER):
 		sprintf(buf, COOKIES_ALLOWED_VIA_PROMPT);
-	    break;
+		break;
+	    case (FROM_FILE):
+		sprintf(buf, COOKIES_READ_FROM_FILE);
+		break;
 	}
 	(*target->isa->put_block)(target, buf, strlen(buf));
+	sprintf(buf, "\n");
+	(*target->isa->put_block)(target, buf, strlen(buf));
 
 	/*
 	 *  Show the domain's cookies. - FM
@@ -2337,6 +2496,162 @@ Delete_all_cookies_in_domain:
     return(HT_LOADED);
 }
 
+
+/*      cookie_add_acceptlist
+**      ---------------------
+**
+**   Is passed a comma-delimited string of domains to add to the
+**   "always accept" list for cookies.  The domains need to be identical
+**   to the form from which the cookie is received, with or without a
+**   leading ".".  - BJP
+*/
+
+PUBLIC void cookie_add_acceptlist ARGS1(
+	char *, 	acceptstr)
+{
+    domain_entry *de = NULL;
+    domain_entry *de2 = NULL;
+    HTList *hl;
+    char **str = (char **)calloc(1, sizeof(acceptstr));
+    char *astr = NULL;
+    char *strsmall = NULL;
+    int isexisting = FALSE;
+
+    /*
+     * Is this the first cookie we're handling?  If so, initialize
+     * domain_list.
+     */
+
+    if (domain_list == NULL) {
+	atexit(LYCookieJar_free);
+	domain_list = HTList_new();
+	total_cookies = 0;
+    }
+
+    StrAllocCopy(astr, acceptstr);
+
+    *str = astr;
+
+    for(; (strsmall = LYstrsep(str, ","));) {
+	if(strsmall == NULL)
+	    break;
+
+	/*
+	 * Check the list of existing cookies to see if this is a
+	 * re-setting of an already existing cookie -- if so, just
+	 * change the behavior, if not, create a new domain entry.
+	 */
+
+	for (hl = domain_list; hl != NULL; hl = hl->next) {
+	    de2 = (domain_entry *)hl->object;
+	    if ((de2 != NULL && de2->domain != NULL) &&
+		!strcmp(strsmall, de2->domain)) {
+			isexisting = TRUE;
+			break;
+	    } else {
+		isexisting = FALSE;
+	    }
+	}
+
+	if(!isexisting) {
+	    de = (domain_entry *)calloc(1, sizeof(domain_entry));
+
+	    if (de == NULL)
+		    outofmem(__FILE__, "cookie_add_acceptlist");
+
+	    de->bv = ACCEPT_ALWAYS;
+
+	    StrAllocCopy(de->domain, strsmall);
+	    de->cookie_list = HTList_new();
+	    HTList_addObject(domain_list, de);
+	} else {
+	    de2->bv = ACCEPT_ALWAYS;
+	}
+    }
+
+    FREE(str);
+    FREE(strsmall);
+    FREE(astr);
+}
+
+
+/*      cookie_add_rejectlist
+**      ---------------------
+**
+**   Is passed a comma-delimited string of domains to add to the
+**   "always reject" list for cookies.  The domains need to be identical
+**   to the form from which the cookie is received, with or without a
+**   leading ".".  - BJP
+*/
+
+PUBLIC void cookie_add_rejectlist ARGS1(
+	char *, 	rejectstr)
+{
+    domain_entry *de = NULL;
+    domain_entry *de2 = NULL;
+    HTList *hl;
+    char **str = (char **)calloc(1, sizeof(rejectstr));
+    char *rstr = NULL;
+    char *strsmall = NULL;
+    int isexisting = FALSE;
+
+    /*
+     * Is this the first cookie we're handling?  If so, initialize
+     * domain_list.
+     */
+
+    if (domain_list == NULL) {
+	atexit(LYCookieJar_free);
+	domain_list = HTList_new();
+	total_cookies = 0;
+    }
+
+    StrAllocCopy(rstr, rejectstr);
+
+    *str = rstr;
+
+    for(; (strsmall = LYstrsep(str, ","));) {
+	if(strsmall == NULL)
+	    break;
+
+	/*
+	 * Check the list of existing cookies to see if this is a
+	 * re-setting of an already existing cookie -- if so, just
+	 * change the behavior, if not, create a new domain entry.
+	 */
+
+	for (hl = domain_list; hl != NULL; hl = hl->next) {
+	    de2 = (domain_entry *)hl->object;
+	    if ((de2 != NULL && de2->domain != NULL) &&
+		!strcmp(strsmall, de2->domain)) {
+			isexisting = TRUE;
+			break;
+	    } else {
+		isexisting = FALSE;
+	    }
+	}
+
+	if(!isexisting) {
+	    de = (domain_entry *)calloc(1, sizeof(domain_entry));
+
+	    if (de == NULL)
+		    outofmem(__FILE__, "cookie_add_rejectlist");
+
+	    de->bv = REJECT_ALWAYS;
+
+	    StrAllocCopy(de->domain, strsmall);
+	    de->cookie_list = HTList_new();
+	    HTList_addObject(domain_list, de);
+	} else {
+	    de2->bv = REJECT_ALWAYS;
+	}
+    }
+
+    FREE(str);
+    FREE(strsmall);
+    FREE(rstr);
+}
+
 #ifdef GLOBALDEF_IS_MACRO
 #define _LYCOOKIE_C_GLOBALDEF_1_INIT { "LYNXCOOKIE",LYHandleCookies,0}
 GLOBALDEF (HTProtocol,LYLynxCookies,_LYCOOKIE_C_GLOBALDEF_1_INIT);
