@@ -5,31 +5,28 @@
 **
 */
 
-#include "HTUtils.h"
-#include "tcp.h"
-#include "HTTP.h"
-#include "HTAnchor.h"
-#include "HTAccess.h"
-#include "HTFormat.h"
-#include "HTParse.h"
-#include "HTAlert.h"
-#include "LYUtils.h"
-#include "LYMap.h"
-#include "GridText.h"
-#include "LYSignal.h"
-#include "LYGlobalDefs.h"
-#include "LYKeymap.h"
-#include "LYCharUtils.h"
+#include <HTUtils.h>
+#include <HTTP.h>
+#include <HTAnchor.h>
+#include <HTAccess.h>
+#include <HTFormat.h>
+#include <HTParse.h>
+#include <HTAlert.h>
+#include <LYUtils.h>
+#include <LYMap.h>
+#include <GridText.h>
+#include <LYSignal.h>
+#include <LYGlobalDefs.h>
+#include <LYKeymap.h>
+#include <LYCharUtils.h>
 
 #ifdef DIRED_SUPPORT
-#include "LYUpload.h"
-#include "LYLocal.h"
+#include <LYUpload.h>
+#include <LYLocal.h>
 #endif
 
-#include "LYexit.h"
-#include "LYLeaks.h"
-
-#define FREE(x) if (x) {free(x); x=NULL;}
+#include <LYexit.h>
+#include <LYLeaks.h>
 
 typedef struct _LYMapElement {
    char * address;
@@ -171,6 +168,8 @@ PUBLIC BOOL LYAddImageMap ARGS3(
     if (theList) {
 	cur = theList;
 	while (NULL != (old = (LYImageMap *)HTList_nextObject(cur))) {
+	    if (old->address == 0)	/* shouldn't happen */
+	    	continue;
 	    if (!strcmp(old->address, address)) {
 		FREE(old->address);
 		FREE(old->title);
