@@ -2,7 +2,7 @@
 #define LYNEWSPOST_H
 
 #ifndef LYSTRUCTS_H
-#include "LYStructs.h"
+#include <LYStructs.h>
 #endif /* LYSTRUCTS_H */
 
 extern BOOLEAN term_message;
