@@ -2,7 +2,7 @@
 #define LYREADCFG_H
 
 #ifndef LYSTRUCTS_H
-#include "LYStructs.h"
+#include <LYStructs.h>
 #endif /* LYSTRUCTS_H */
 
 #if defined(USE_COLOR_STYLE) || defined(USE_COLOR_TABLE)
@@ -43,7 +43,10 @@ extern int default_bg;
 extern int check_color PARAMS((char * color, int the_default));
 #endif
 
-extern void read_cfg PARAMS((char *cfg_filename));
+extern void read_cfg PARAMS((char *cfg_filename, char *parent_filename, int nesting_level, FILE *fp0));
+extern void free_lynx_cfg NOPARAMS;
 extern BOOLEAN have_read_cfg;
 
+extern char *lynx_cfg_infopage NOPARAMS;
+
 #endif /* LYREADCFG_H */
