@@ -1,25 +1,25 @@
-#include "HTUtils.h"
-#include "tcp.h"
-#include "HTCJK.h"
-#include "LYCurses.h"
-#include "LYUtils.h"
-#include "LYStrings.h"
-#include "LYGlobalDefs.h"
-#include "GridText.h"
-#include "LYKeymap.h"
-#include "LYSignal.h"
-#include "LYClean.h"
-#include "LYMail.h"
-#include "LYNews.h"
-#include "LYOptions.h"
-#include "LYCharSets.h"
-#include "HTString.h"
+#include <HTUtils.h>
+#include <HTCJK.h>
+#include <LYCurses.h>
+#include <LYUtils.h>
+#include <LYStrings.h>
+#include <LYGlobalDefs.h>
+#include <GridText.h>
+#include <LYKeymap.h>
+#include <LYSignal.h>
+#include <LYClean.h>
+#include <LYMail.h>
+#include <LYNews.h>
+#include <LYOptions.h>
+#include <LYCharSets.h>
+#include <HTString.h>
 
-#include <ctype.h>
+#ifdef DJGPP_KEYHANDLER
+#include <pc.h>
+#include <keys.h>
+#endif /* DJGPP_KEYHANDLER */
 
-#include "LYLeaks.h"
-
-#define FREE(x) if (x) {free(x); x = NULL;}
+#include <LYLeaks.h>
 
 extern BOOL HTPassHighCtrlRaw;
 extern HTCJKlang HTCJK;
@@ -56,41 +56,47 @@ PUBLIC int get_mouse_link NOARGS
 
 PRIVATE int set_clicked_link ARGS2(int,x,int,y)
 {
-  int i;
+    int left = 6;
+    int right = LYcols-6;
+    /* yes, I am assuming that my screen will be a certain width. */
+    int i;
+    int c = -1;
 
-  /* Loop over the links and see if we can get a match */
-  for(i=0; i < nlinks && mouse_link == -1; i++) {
-    /* Check the first line of the link */
-    if ( links[i].hightext != NULL &&
-	 links[i].ly == y &&
-	 (x - links[i].lx) < (int)strlen(links[i].hightext ) ) {
-      mouse_link=i;
+    if (y == (LYlines-1)) {
+	if (x < left) c = LTARROW;
+	else if (x > right) c = '\b';
+	else c = PGDOWN;
+    } else if (y == 0) {
+	if (x < left) c = LTARROW;
+	else if (x > right) c = '\b';
+	else c = PGUP;
+    } else {
+	/* Loop over the links and see if we can get a match */
+	for (i = 0; i < nlinks; i++) {
+	    /* Check the first line of the link */
+	    if ( links[i].hightext != NULL &&
+		links[i].ly == y &&
+		(x - links[i].lx) < (int)strlen(links[i].hightext ) ) {
+		mouse_link = i;
+		break;
+	    }
+	    /* Check the second line */
+	    if (links[i].hightext2 != NULL &&
+		1+links[i].ly == y &&
+		(x - links[i].hightext2_offset) < (int)strlen(links[i].hightext2) ) {
+		mouse_link = i;
+		break;
+	    }
+	}
+	/*
+	 * If a link was hit, we must look for a key which will activate
+	 * LYK_ACTIVATE We expect to find LYK_ACTIVATE (it's usually mapped to
+	 * the Enter key).
+	 */
+	if (mouse_link >= 0)
+	    c = lookup_keymap(LYK_ACTIVATE);
     }
-    /* Check the second line */
-    if (links[i].hightext2 != NULL &&
-	1+links[i].ly == y &&
-	(x - links[i].hightext2_offset) < (int)strlen(links[i].hightext2) ) {
-      mouse_link=i;
-    }
-  }
-  /* If no link was found, just return a do-nothing code */
-  if (mouse_link == -1) return -1;
-
-  /* If a link was hit, we must look for a key which will activate LYK_ACTIVATE
-  ** XXX The 127 in the following line will depend on the size of the keymap[]
-  ** array.  However, usually we'll find LYK_ACTIVATE somewhere in the first
-  ** 127 keys (it's usually mapped to the Enter key)
-  **/
-  for (i=0; i<127; i++) {
-    if (LYisNonAlnumKeyname(i, LYK_ACTIVATE)) {
-      return i;
-    }
-  }
-    /* Whoops!	Nothing's defined as LYK_ACTIVATE!
-       Well, who are we to argue with the user?
-       Forget about the mouse click */
-  mouse_link = -1;
-  return -1;
+    return c;
 }
 
 
@@ -104,7 +110,7 @@ PUBLIC char *LYstrncpy ARGS3(
 	int,		n)
 {
     char *val;
-    int len=strlen(src);
+    int len = strlen(src);
 
     if (n < 0)
 	n = 0;
@@ -232,7 +238,15 @@ PUBLIC int LYmbcsstrlen ARGS2(
 #ifdef VMS
 #define GetChar() ttgetc()
 #else
+#ifdef __DJGPP__
+#ifdef DJGPP_KEYHANDLER
+#define GetChar getxkey
+#else
+#define GetChar SLkp_getkey
+#endif /* DJGPP_KEYHANDLER */
+#else
 #define GetChar (int)SLang_getkey
+#endif /* __DJGPP__ */
 #endif /* VMS */
 #endif /* USE_SLANG */
 
@@ -263,53 +277,62 @@ PUBLIC int LYmbcsstrlen ARGS2(
 /*
  * Workaround a bug in ncurses order-of-refresh by setting a pointer to
  * the topmost window that should be displayed.
+ *
+ * FIXME: the associated call on 'keypad()' is not needed for Unix, but
+ * something in the OS/2 EMX port requires it.
  */
 PRIVATE WINDOW *my_subwindow;
 
 PUBLIC void LYsubwindow ARGS1(WINDOW *, param)
 {
-	my_subwindow = param;
+    if ((my_subwindow = param) != 0)
+	keypad(param, TRUE);
 }
 #endif
 
 #ifdef USE_SLANG_MOUSE
 PRIVATE int sl_parse_mouse_event ARGS3(int *, x, int *, y, int *, button)
 {
-   /* "ESC [ M" has already been processed.  There more characters are
-    * expected:  BUTTON X Y
-    */
-   *button = SLang_getkey ();
-   switch (*button)
-     {
-      case 040: 		       /* left button */
-      case 041: 		       /* middle button */
-      case 042: 		       /* right button */
+    /* "ESC [ M" has already been processed.  There more characters are
+     * expected:  BUTTON X Y
+     */
+    *button = SLang_getkey ();
+    switch (*button)
+    {
+    case 040:			/* left button */
+    case 041:			/* middle button */
+    case 042:			/* right button */
 	*button -= 040;
 	break;
 
-      default:			       /* Hmmm.... */
+    default:			/* Hmmm.... */
 	SLang_flush_input ();
 	return -1;
-     }
+    }
 
-   *x = SLang_getkey () - 33;
-   *y = SLang_getkey () - 33;
-   return 0;
+    *x = SLang_getkey () - 33;
+    *y = SLang_getkey () - 33;
+    return 0;
 }
-#endif
 
-#if defined(USE_SLANG_MOUSE) || defined(NCURSES_MOUSE_VERSION)
-PRIVATE int map_function_to_key ARGS1(char, keysym)
+PRIVATE int sl_read_mouse_event NOARGS
 {
-   int i;
+   int mouse_x, mouse_y, button;
 
-   /* I would prefer to use sizeof keymap but its size is not available.
-    * A better method would be to declare it as some fixed size.
-    */
-   for (i = 1; i < 256; i++)
+   mouse_link = -1;
+   if (-1 != sl_parse_mouse_event (&mouse_x, &mouse_y, &button))
      {
-	if (keymap[i] == keysym)
-	  return i - 1;
+	if (button == 0)  /* left */
+	  return set_clicked_link (mouse_x, mouse_y);
+
+	if (button == 2)   /* right */
+	  {
+	     /* Right button: go back to prev document.
+	      * The problem is that we need to determine
+	      * what to return to achieve this.
+	      */
+	     return LYReverseKeymap (LYK_PREV_DOC);
+	  }
      }
    return -1;
 }
@@ -321,9 +344,477 @@ PUBLIC void ena_csi ARGS1(
 {
     csi_is_csi = flag;
 }
+
+#if defined(USE_KEYMAPS)
+
+#ifdef USE_SLANG
+#define define_key(string, code) \
+	SLkm_define_keysym (string, code, Keymap_List)
+#define expand_substring(dst, first, last) \
+	SLexpand_escaped_string(dst, first, last)
+static SLKeyMap_List_Type *Keymap_List;
+/* This value should be larger than anything in LYStrings.h */
+#define MOUSE_KEYSYM 0x1000
+#endif
+
+
+#define SQUOTE '\''
+#define DQUOTE '"'
+#define ESCAPE '\\'
+#define LPAREN '('
+#define RPAREN ')'
+
+/*
+ * For ncurses, we use the predefined keysyms, since that lets us also reuse
+ * the CSI logic and other special cases for VMS, NCSA telnet, etc.
+ */
+#ifdef USE_SLANG
+#define DEFINE_KEY(string,lynx,curses) {string,lynx}
+#else
+#define DEFINE_KEY(string,lynx,curses) {string,curses}
+#endif
+
+typedef struct
+{
+   char *string;
+   int value;
+}
+Keysym_String_List;
+
+static Keysym_String_List Keysym_Strings [] =
+{
+    DEFINE_KEY( "UPARROW",	UPARROW,	KEY_UP ),
+    DEFINE_KEY( "DNARROW",	DNARROW,	KEY_DOWN ),
+    DEFINE_KEY( "RTARROW",	RTARROW,	KEY_RIGHT ),
+    DEFINE_KEY( "LTARROW",	LTARROW,	KEY_LEFT ),
+    DEFINE_KEY( "PGDOWN",	PGDOWN,		KEY_NPAGE ),
+    DEFINE_KEY( "PGUP",		PGUP,		KEY_PPAGE ),
+    DEFINE_KEY( "HOME",		HOME,		KEY_HOME ),
+    DEFINE_KEY( "END",		END_KEY,	KEY_END ),
+    DEFINE_KEY( "F1",		F1,		KEY_F(1) ),
+    DEFINE_KEY( "DO_KEY",	DO_KEY,		KEY_F(16) ),
+    DEFINE_KEY( "FIND_KEY",	FIND_KEY,	KEY_FIND ),
+    DEFINE_KEY( "SELECT_KEY",	SELECT_KEY,	KEY_SELECT ),
+    DEFINE_KEY( "INSERT_KEY",	INSERT_KEY,	KEY_IC ),
+    DEFINE_KEY( "REMOVE_KEY",	REMOVE_KEY,	KEY_DC ),
+    DEFINE_KEY( "DO_NOTHING",	DO_NOTHING,	0 ),
+    DEFINE_KEY( NULL, 		-1,		ERR )
+};
+
+#ifdef NCURSES_VERSION
+/*
+ * Ncurses stores the termcap/terminfo names in arrays sorted to match the
+ * array of strings in the TERMTYPE struct.
+ */
+PRIVATE int lookup_tiname (char *name, NCURSES_CONST char *const *names)
+{
+    int code;
+
+    for (code = 0; names[code] != 0; code++)
+	if (!strcmp(names[code], name))
+	    return code;
+    return -1;
+}
+
+PRIVATE char *expand_tiname (char *first, size_t len, char **result)
+{
+    char name[BUFSIZ];
+    int code;
+
+    strncpy(name, first, len);
+    name[len] = '\0';
+    if ((code = lookup_tiname(name, strnames)) >= 0
+     || (code = lookup_tiname(name, strfnames)) >= 0) {
+	strcpy(*result, cur_term->type.Strings[code]);
+	(*result) += strlen(*result);
+    }
+    return first + len;
+}
+
+PRIVATE char *expand_tichar (char *first, char **result)
+{
+    int ch;
+    int limit = 0;
+    int radix = 0;
+    int value = 0;
+    char *name = 0;
+
+    switch (ch = *first++) {
+    case 'E': case 'e':	value = 27;			break;
+    case 'a':		name  = "bel";			break;
+    case 'b':		value = '\b';			break;
+    case 'f':		value = '\f';			break;
+    case 'n':		value = '\n';			break;
+    case 'r':		value = '\r';			break;
+    case 't':		value = '\t';			break;
+    case 'v':		value = '\v';			break;
+    case 'd':		radix = 10;	limit = 3;	break;
+    case 'x':		radix = 16;	limit = 2;	break;
+    default:
+	if (isdigit(ch)) {
+	    radix = 8;
+	    limit = 3;
+	    first--;
+	} else {
+	    value = *first;
+	}
+	break;
+    }
+
+    if (radix != 0) {
+	char *last;
+	int save = first[limit];
+	first[limit] = '\0';
+	value = strtol(first, &last, radix);
+	first[limit] = save;
+	first = last;
+    }
+
+    if (name != 0) {
+	(void) expand_tiname(name, strlen(name), result);
+    } else {
+	**result = value;
+	(*result) += 1;
+    }
+
+    return first;
+}
+
+PRIVATE void expand_substring (char* dst, char* first, char* last)
+{
+    int ch;
+    while (first < last) {
+	switch (ch = *first++) {
+	case ESCAPE:
+	    first = expand_tichar(first, &dst);
+	    break;
+	case '^':
+	    ch = *first++;
+	    if (ch == LPAREN) {
+		char *s = strchr(first, RPAREN);
+		if (s == 0)
+		    s = first + strlen(first);
+		first = expand_tiname(first, s-first, &dst);
+	    } else if (ch == '?') {		/* ASCII delete? */
+		*dst++ = 127;
+	    } else if ((ch & 0x3f) < 0x20) {	/* ASCII control char? */
+		*dst++ = (ch & 0x1f);
+	    } else {
+		*dst++ = '^';
+		first--;	/* not legal... */
+	    }
+	    break;
+	case 0:					/* convert nulls for terminfo */
+	    ch = 0200;
+	    /* FALLTHRU */
+	default:
+	    *dst++ = ch;
+	    break;
+	}
+    }
+    *dst = '\0';
+}
+#endif
+
+PRIVATE void unescaped_char ARGS2(char*, parse, int*,keysym)
+{
+    size_t len = strlen(parse);
+    char buf[BUFSIZ];
+
+    if (len >= 3) {
+	expand_substring(buf, parse + 1, parse + len - 1);
+	if (strlen(buf) == 1)
+	    *keysym = *buf;
+    }
+}
+
+PRIVATE BOOLEAN unescape_string ARGS2(char*, src, char *, dst)
+{
+    BOOLEAN ok = FALSE;
+
+    if (*src == SQUOTE) {
+	int keysym;
+	unescaped_char(src, &keysym);
+	if (keysym >= 0) {
+	    dst[0] = keysym;
+	    dst[1] = '\0';
+	    ok = TRUE;
+	}
+    } else if (*src == DQUOTE) {
+	expand_substring(dst, src + 1, src + strlen(src) - 1);
+	ok = TRUE;
+    }
+    return ok;
+}
+
+PRIVATE int map_string_to_keysym ARGS2(char*, str, int*,keysym)
+{
+    *keysym = -1;
+
+    if (*str == SQUOTE) {
+	unescaped_char(str, keysym);
+    } else if (isdigit(*str)) {
+	char *tmp;
+	long value = strtol(str, &tmp, 0);
+	if (!isalnum(*tmp))
+	    *keysym = value;
+    } else {
+	Keysym_String_List *k;
+
+	k = Keysym_Strings;
+	while (k->string != NULL) {
+	    if (0 == strcmp (k->string, str)) {
+		*keysym = k->value;
+		break;
+	    }
+	    k++;
+	}
+    }
+
+    return (*keysym);
+}
+
+/*
+ * Starting at a nonblank character, skip over a token, counting quoted and
+ * escaped characters. 
+ */
+PRIVATE char *skip_keysym ARGS1(char *, parse)
+{
+    int quoted = 0;
+    int escaped = 0;
+
+    while (*parse) {
+	if (escaped) {
+	    escaped = 0;
+	} else if (quoted) {
+	    if (*parse == ESCAPE) {
+		escaped = 1;
+	    } else if (*parse == quoted) {
+		quoted = 0;
+	    }
+	} else if (*parse == ESCAPE) {
+	    escaped = 1;
+	} else if (*parse == DQUOTE || *parse == SQUOTE) {
+	    quoted = *parse;
+	} else if (isspace(*parse)) {
+	    break;
+	}
+	parse++;
+    }
+    return (quoted || escaped) ? 0 : parse;
+}
+
+/*
+ * The first token is the string to define, the second is the name (of the
+ * keysym) to define it to.
+ */
+PRIVATE int setkey_cmd (char *parse)
+{
+    char *s, *t;
+    int keysym;
+    char buf[BUFSIZ];
+
+    if ((s = skip_keysym(parse)) != 0) {
+	if (isspace(*s)) {
+	    *s++ = '\0';
+	    s = LYSkipBlanks(s);
+	    if ((t = skip_keysym(s)) == 0)
+		return -1;
+	    if (t != s)
+		*t = '\0';
+	    if (map_string_to_keysym (s, &keysym) >= 0
+	     && unescape_string(parse, buf)) {
+		return define_key(buf, keysym);
+	    }
+	}
+    }
+    return -1;
+}
+
+PRIVATE int unsetkey_cmd (char *parse)
+{
+    char *s = skip_keysym(parse);
+    if (s != parse) {
+	*s = '\0';
+#ifdef NCURSES_VERSION
+	/*
+	 * This won't work with Slang.  Remove the definition for the given
+	 * keysym.
+	 */
+	{
+	    int keysym;
+	    if (map_string_to_keysym (parse, &keysym) >= 0)
+		define_key((char *)0, keysym);
+	}
+#endif
+#ifdef USE_SLANG
+	/* Slang implements this, for undefining the string which is associated
+	 * with a keysym (the reverse of what we normally want, but may
+	 * occasionally find useful).
+	 */
+	SLang_undefine_key (parse, Keymap_List);
+	if (SLang_Error) return -1;
+#endif
+    }
+    return 0;
+}
+
+#ifdef FNAMES_8_3
+#define FNAME_LYNX_KEYMAPS "_lynxkey.map"
+#else
+#define FNAME_LYNX_KEYMAPS ".lynx-keymaps"
+#endif /* FNAMES_8_3 */
+
+PRIVATE int read_keymap_file NOARGS
+{
+    static struct {
+	CONST char *name;
+	int (*func) PARAMS((char *s));
+    } table[] = {
+	{"setkey",   setkey_cmd },
+	{"unsetkey", unsetkey_cmd },
+    };
+
+    char line[1024];
+    FILE *fp;
+    char file[LY_MAXPATH];
+    int ret;
+    int linenum;
+    size_t n;
+
+    LYAddPathToHome(file, sizeof(file), FNAME_LYNX_KEYMAPS);
+
+    if ((fp = fopen (file, "r")) == 0)
+	return 0;
+
+    linenum = 0;
+    ret = 0;
+    while ((fgets (line, sizeof (line), fp) != 0) && (ret == 0))
+    {
+	char *s = LYSkipBlanks(line);
+
+	linenum++;
+
+	if ((*s == 0) || (*s == '#'))
+	    continue;
+
+	for (n = 0; n < TABLESIZE(table); n++) {
+	    size_t len = strlen(table[n].name);
+	    if (strlen(s) > len
+	     && !strncmp(s, table[n].name, len)) {
+		if ((*(table[n].func))(LYSkipBlanks(s+len)) < 0) {
+		    ret = -1;
+		    break;
+		}
+	    }
+	}
+    }
+
+    fclose (fp);
+
+    if (ret == -1)
+	fprintf (stderr, "Error processing line %d of %s\n", linenum, file);
+
+    return ret;
+}
+
+PRIVATE void setup_vtXXX_keymap NOARGS
+{
+    static Keysym_String_List table[] = {
+	DEFINE_KEY( "\033[A",	UPARROW,	KEY_UP ),
+	DEFINE_KEY( "\033OA",	UPARROW,	KEY_UP ),
+	DEFINE_KEY( "\033[B",	DNARROW,	KEY_DOWN ),
+	DEFINE_KEY( "\033OB",	DNARROW,	KEY_DOWN ),
+	DEFINE_KEY( "\033[C",	RTARROW,	KEY_RIGHT ),
+	DEFINE_KEY( "\033OC",	RTARROW,	KEY_RIGHT ),
+	DEFINE_KEY( "\033[D",	LTARROW,	KEY_LEFT ),
+	DEFINE_KEY( "\033OD",	LTARROW,	KEY_LEFT ),
+	DEFINE_KEY( "\033[1~",	FIND_KEY,	KEY_FIND ),
+	DEFINE_KEY( "\033[2~",	INSERT_KEY,	KEY_IC ),
+	DEFINE_KEY( "\033[3~",	REMOVE_KEY,	KEY_DC ),
+	DEFINE_KEY( "\033[4~",	SELECT_KEY,	KEY_SELECT ),
+	DEFINE_KEY( "\033[5~",	PGUP,		KEY_PPAGE ),
+	DEFINE_KEY( "\033[6~",	PGDOWN,		KEY_NPAGE ),
+	DEFINE_KEY( "\033[8~",	END_KEY,	KEY_END ),
+	DEFINE_KEY( "\033[7~",	HOME,		KEY_HOME),
+	DEFINE_KEY( "\033[28~",	F1,		KEY_F(1) ),
+	DEFINE_KEY( "\033OP",	F1,		KEY_F(1) ),
+	DEFINE_KEY( "\033[OP",	F1,		KEY_F(1) ),
+	DEFINE_KEY( "\033[29~",	DO_KEY,		KEY_F(16) ),
+    };
+    size_t n;
+    for (n = 0; n < TABLESIZE(table); n++)
+    	define_key(table[n].string, table[n].value);
+}
+
+PUBLIC int lynx_initialize_keymaps NOARGS
+{
+#ifdef USE_SLANG
+    int i;
+    char keybuf[2];
+
+    if (NULL == (Keymap_List = SLang_create_keymap ("Lynx", NULL)))
+	return -1;
+
+    keybuf[1] = 0;
+    for (i = 1; i < 256; i++)
+    {
+	keybuf[0] = (char) i;
+	define_key (keybuf, i);
+    }
+
+    setup_vtXXX_keymap();
+    define_key ("\033[M", MOUSE_KEYSYM);
+
+    if (SLang_Error
+    || (-1 == read_keymap_file ()))
+    SLang_exit_error ("Unable to initialize keymaps");
+    return 0;
+#else
+    setup_vtXXX_keymap();
+    return read_keymap_file();
+#endif
+}
+
+#endif				       /* USE_KEYMAPS */
+
+#if defined(USE_KEYMAPS) && defined(USE_SLANG)
+
+/* We cannot guarantee the type for 'GetChar', and should not use a cast. */
+PRIVATE int myGetChar NOARGS
+{
+   return GetChar();
+}
+
+PUBLIC int LYgetch NOARGS
+{
+   SLang_Key_Type *key;
+   int keysym;
+
+   key = SLang_do_key (Keymap_List, myGetChar);
+   if ((key == NULL) || (key->type != SLKEY_F_KEYSYM))
+     return DO_NOTHING;
+
+   keysym = key->f.keysym;
+
+#if defined (USE_SLANG_MOUSE)
+   if (keysym == MOUSE_KEYSYM)
+     return sl_read_mouse_event ();
+#endif
+
+   if ((keysym > DO_NOTHING) || (keysym < 0))
+     return 0;
+
+   return keysym;
+}
+
+#else /* !USE_KEYMAPS */
+
 /*
  *  LYgetch() translates some escape sequences and may fake noecho.
  */
+#define found_CSI(first,second) ((second) == '[' || (first) == 155)
+
 PUBLIC int LYgetch NOARGS
 {
     int a, b, c, d = -1;
@@ -334,7 +825,7 @@ re_read:
 #ifndef USE_SLANG
     clearerr(stdin); /* needed here for ultrix and SOCKETSHR, but why? - FM */
 #endif /* !USE_SLANG */
-#if !defined(USE_SLANG) || defined(VMS)
+#if !defined(USE_SLANG) || defined(VMS) || defined(DJGPP_KEYHANDLER)
     c = GetChar();
 #else
     if (LYCursesON) {
@@ -350,26 +841,8 @@ re_read:
 #endif /* IGNORE_CTRL_C */
 	    return(7); /* use ^G to cancel whatever called us. */
 	}
-   }
-#endif /* !USE_SLANG || VMS */
-
-#ifdef RAWDOSKEYHACK
-    if (raw_dos_key_hack) {
-	if (c == 0) c = '/';
-	if (c > 255) {	    /* handle raw dos keys */
-	    switch (c)
-	    {
-		case 464: c = '-';	break;	/* keypad minus*/
-		case 465: c = '+';	break;	/* keypad plus*/
-		case 459: c = 13;	break;	/* keypad enter*/
-		case 463: c = '*';	break;	/* keypad * */
-		case 440: c = 'Q';	break;	/* alt x */
-		case 265: c = 'H';	break;	/* F1 */
-		default: break;
-	    }
-	}
     }
-#endif /* RAWDOSKEYHACK */
+#endif /* !USE_SLANG || VMS */
 
 #ifdef USE_GETCHAR
     if (c == EOF && errno == EINTR)	/* Ctrl-Z causes EINTR in getchar() */
@@ -435,32 +908,16 @@ re_read:
 	case 'C': c = RTARROW; break;
 	case 'v': c = RTARROW; break; /* keypad right on pc ncsa telnet */
 	case 'D': c = LTARROW; break;
-	case 't': c = LTARROW; break; /* keypad left on pc ncsa telnet */
-	case 'y': c = PGUP; break;  /* keypad on pc ncsa telnet */
-	case 's': c = PGDOWN; break;  /* keypad on pc ncsa telnet */
-	case 'w': c = HOME; break;  /* keypad on pc ncsa telnet */
-	case 'q': c = END; break;  /* keypad on pc ncsa telnet */
+	case 't': c = LTARROW; break;  /* keypad left on pc ncsa telnet */
+	case 'y': c = PGUP;    break;  /* keypad on pc ncsa telnet */
+	case 's': c = PGDOWN;  break;  /* keypad on pc ncsa telnet */
+	case 'w': c = HOME;    break;  /* keypad on pc ncsa telnet */
+	case 'q': c = END_KEY; break;  /* keypad on pc ncsa telnet */
 	case 'M':
 #ifdef USE_SLANG_MOUSE
-	   if ((c == 27) && (b == '['))
+	   if (found_CSI(c,b))
 	     {
-		int mouse_x, mouse_y, button;
-
-		mouse_link = -1;
-		c = -1;
-		if (-1 != sl_parse_mouse_event (&mouse_x, &mouse_y, &button))
-		  {
-		     if (button == 0)  /* left */
-		       c = set_clicked_link (mouse_x, mouse_y);
-		     else if (button == 2)   /* right */
-		       {
-			  /* Right button: go back to prev document.
-			   * The problem is that we need to determine
-			   * what to return to achieve this.
-			   */
-			  c = map_function_to_key (LYK_PREV_DOC);
-		       }
-		  }
+		c = sl_read_mouse_event ();
 	     }
 	   else
 #endif
@@ -502,11 +959,11 @@ re_read:
 		c = '0';  /* keypad 0 */
 	    break;
 	case '1':			    /** VTxxx  Find  **/
-	    if ((b == '[' || c == 155) && (d=GetChar()) == '~')
+	    if (found_CSI(c,b) && (d=GetChar()) == '~')
 		c = FIND_KEY;
 	    break;
 	case '2':
-	    if (b == '[' || c == 155) {
+	    if (found_CSI(c,b)) {
 		if ((d=GetChar())=='~')     /** VTxxx Insert **/
 		    c = INSERT_KEY;
 		else if ((d == '8' ||
@@ -522,36 +979,33 @@ re_read:
 	    }
 	    break;
 	case '3':			     /** VTxxx Delete **/
-	    if ((b == '[' || c == 155) && (d=GetChar()) == '~')
+	    if (found_CSI(c,b) && (d=GetChar()) == '~')
 		c = REMOVE_KEY;
 	    break;
 	case '4':			     /** VTxxx Select **/
-	    if ((b == '[' || c == 155) && (d=GetChar()) == '~')
+	    if (found_CSI(c,b) && (d=GetChar()) == '~')
 		c = SELECT_KEY;
 	    break;
 	case '5':			     /** VTxxx PrevScreen **/
-	    if ((b == '[' || c == 155) && (d=GetChar()) == '~')
+	    if (found_CSI(c,b) && (d=GetChar()) == '~')
 		c = PGUP;
 	    break;
 	case '6':			     /** VTxxx NextScreen **/
-	    if ((b == '[' || c == 155) && (d=GetChar()) == '~')
+	    if (found_CSI(c,b) && (d=GetChar()) == '~')
 		c = PGDOWN;
 	    break;
 	case '[':			     /** Linux F1-F5: ^[[[A etc. **/
-	    if (b == '[' || c == 155) {
+	    if (found_CSI(c,b)) {
 		if ((d=GetChar()) == 'A')
 		    c = F1;
 		break;
 	    }
 	default:
-	   if (TRACE) {
-		fprintf(stderr,"Unknown key sequence: %d:%d:%d\n",c,b,a);
-		if (!LYTraceLogFP) {
-		    sleep(MessageSecs);
-		}
-	   }
+	    CTRACE(tfp,"Unknown key sequence: %d:%d:%d\n",c,b,a);
+	    CTRACE_SLEEP(MessageSecs);
+	    break;
 	}
-	if (isdigit(a) && (b == '[' || c == 155) && d != -1 && d != '~')
+	if (isdigit(a) && found_CSI(c,b) && d != -1 && d != '~')
 	    d = GetChar();
     }
 #if HAVE_KEYPAD
@@ -585,7 +1039,7 @@ re_read:
 	   c = PGUP;
 	   break;
 	case KEY_LL:		   /* home down or bottom (lower left) */
-	   c = END;
+	   c = END_KEY;
 	   break;
 					/* The keypad is arranged like this:*/
 					/*    a1    up	  a3   */
@@ -601,14 +1055,14 @@ re_read:
 	   c = DO_NOTHING;
 	   break;
 	case KEY_C1:		   /* lower left of keypad */
-	   c = END;
+	   c = END_KEY;
 	   break;
 	case KEY_C3:		   /* lower right of keypad */
 	   c = PGDOWN;
 	   break;
 #ifdef KEY_END
 	case KEY_END:		   /* end key		001 */
-	   c = END;
+	   c = END_KEY;
 	   break;
 #endif /* KEY_END */
 #ifdef KEY_HELP
@@ -621,7 +1075,7 @@ re_read:
 	   c = 127;		   /* backspace key (delete, not Ctrl-H) */
 	   break;
 #endif /* KEY_BACKSPACE */
-#ifdef KEY_F
+#if defined(KEY_F) && !defined(__DJGPP__) && !defined(_WINDOWS)
 	case KEY_F(1):
 	   c = F1;		   /* VTxxx Help */
 	   break;
@@ -656,47 +1110,114 @@ re_read:
 #endif /* KEY_DC */
 #ifdef NCURSES_MOUSE_VERSION
 	case KEY_MOUSE:
-	  {
+	    {
 #ifndef DOSPATH
-	   MEVENT event;
-	   int err;
+		MEVENT event;
+		int err;
 
-	   c = -1;
-	   mouse_link = -1;
-	   err=getmouse(&event);
-	   if (event.bstate & BUTTON1_CLICKED) {
-	     c = set_clicked_link(event.x, event.y);
-	   } else if (event.bstate & BUTTON3_CLICKED) {
-	     c = map_function_to_key (LYK_PREV_DOC);
-	   }
+		c = -1;
+		mouse_link = -1;
+		err = getmouse(&event);
+		if (event.bstate & BUTTON1_CLICKED) {
+		    c = set_clicked_link(event.x, event.y);
+		} else if (event.bstate & BUTTON3_CLICKED) {
+		    c = LYReverseKeymap (LYK_PREV_DOC);
+		}
 #else /* pdcurses version */
-	      int left,right;
-	      /* yes, I am assuming that my screen will be a certain width. */
-	      left = 6;
-	      right = LYcols-6;
-	      c = -1;
-	      mouse_link = -1;
-	      request_mouse_pos();
-	      if (Mouse_status.button[0] & BUTTON_CLICKED) {
-		if (Mouse_status.y == (LYlines-1))
-		       if (Mouse_status.x < left) c=LTARROW;
-		       else if (Mouse_status.x > right) c='\b';
-		       else c=PGDOWN;
-		else if (Mouse_status.y == 0)
-		       if (Mouse_status.x < left) c=LTARROW;
-		       else if (Mouse_status.x > right) c='\b';
-		       else c=PGUP;
-		else c = set_clicked_link(Mouse_status.x, Mouse_status.y);
-	      }
-#endif /* _WINDOWS */
-	  }
-	  break;
+		c = -1;
+		mouse_link = -1;
+		request_mouse_pos();
+		if (BUTTON_STATUS(1) & BUTTON_CLICKED) {
+		    c = set_clicked_link(MOUSE_X_POS, MOUSE_Y_POS);
+		} else if (BUTTON_STATUS(3) & BUTTON_CLICKED) {
+		    c = LYReverseKeymap (LYK_PREV_DOC);
+		}
+#endif /* DOSPATH */
+	    }
+	    break;
 #endif /* NCURSES_MOUSE_VERSION */
 	}
     }
 #endif /* HAVE_KEYPAD */
+#ifdef DJGPP_KEYHANDLER
+    else {
+	switch(c) {
+	case K_Down:		   /* The four arrow keys ... */
+	case K_EDown:
+	   c = DNARROW;
+	   break;
+	case K_Up:
+	case K_EUp:
+	   c = UPARROW;
+	   break;
+	case K_Left:
+	case K_ELeft:
+	   c = LTARROW;
+	   break;
+	case K_Right: 		   /* ... */
+	case K_ERight:
+	   c = RTARROW;
+	   break;
+	case K_Home:		   /* Home key (upward+left arrow) */
+	case K_EHome:
+	   c = HOME;
+	   break;
+	case K_PageDown: 	   /* Next page */
+	case K_EPageDown:
+	   c = PGDOWN;
+	   break;
+	case K_PageUp:	 	   /* Previous page */
+	case K_EPageUp:
+	   c = PGUP;
+	   break;
+	case K_End:		   /* home down or bottom (lower left) */
+	case K_EEnd:
+	   c = END_KEY;
+	   break;
+	}
+    }
+#endif /* DGJPP_KEYHANDLER */
+#if defined(USE_SLANG) && defined(__DJGPP__) && !defined(DJGPP_KEYHANDLER)  && !defined(USE_KEYMAPS)
+    else {
+	switch(c) {
+	case SL_KEY_DOWN:	   /* The four arrow keys ... */
+	   c = DNARROW;
+	   break;
+	case SL_KEY_UP:
+	   c = UPARROW;
+	   break;
+	case SL_KEY_LEFT:
+	   c = LTARROW;
+	   break;
+	case SL_KEY_RIGHT: 	   /* ... */
+	   c = RTARROW;
+	   break;
+	case SL_KEY_HOME:	   /* Home key (upward+left arrow) */
+	case SL_KEY_A1:		   /* upper left of keypad */
+	   c = HOME;
+	   break;
+	case SL_KEY_NPAGE: 	   /* Next page */
+	case SL_KEY_C3:		   /* lower right of keypad */
+	   c = PGDOWN;
+	   break;
+	case SL_KEY_PPAGE: 	   /* Previous page */
+	case SL_KEY_A3:		   /* upper right of keypad */
+	   c = PGUP;
+	   break;
+	case SL_KEY_END:	   /* home down or bottom (lower left) */
+	case SL_KEY_C1:		   /* lower left of keypad */
+	   c = END_KEY;
+	   break;
+	}
+    }
+#endif /* USE_SLANG && __DJGPP__ && !DJGPP_KEYHANDLER && !USE_KEYMAPS */
 
-    if (c > DO_NOTHING) {
+#if (defined(__DJGPP__) || defined(_WINDOWS))
+    if (c > 659)
+#else
+    if (c > DO_NOTHING)
+#endif /* __DJGPP__ || _WINDOWS */
+    {
 	/*
 	 *  Don't return raw values for KEYPAD symbols which we may have
 	 *  missed in the switch above if they are obviously invalid when
@@ -708,6 +1229,110 @@ re_read:
     }
 }
 
+#endif				       /* NOT USE_KEYMAPS */
+
+/*
+ * Convert a null-terminated string to lowercase
+ */
+PUBLIC void LYLowerCase ARGS1(
+	char *, 	buffer)
+{
+    size_t i;
+    for (i = 0; buffer[i]; i++)
+	buffer[i] = TOLOWER(buffer[i]);
+}
+
+/*
+ * Convert a null-terminated string to uppercase
+ */
+PUBLIC void LYUpperCase ARGS1(
+	char *, 	buffer)
+{
+    size_t i;
+    for (i = 0; buffer[i]; i++)
+	buffer[i] = TOUPPER(buffer[i]);
+}
+
+/*
+ * Remove ALL whitespace from a string (including embedded blanks).
+ */
+PUBLIC void LYRemoveBlanks ARGS1(
+	char *, 	buffer)
+{
+    if (buffer != 0) {
+	size_t i, j;
+	for (i = j = 0; buffer[i]; i++)
+	    if (!isspace((unsigned char)(buffer[i])))
+		buffer[j++] = buffer[i];
+	buffer[j] = 0;
+    }
+}
+
+/*
+ * Skip whitespace
+ */
+PUBLIC char * LYSkipBlanks ARGS1(
+	char *, 	buffer)
+{
+    while (isspace((unsigned char)(*buffer)))
+	buffer++;
+    return buffer;
+}
+
+/*
+ * Skip non-whitespace
+ */
+PUBLIC char * LYSkipNonBlanks ARGS1(
+	char *, 	buffer)
+{
+    while (*buffer != 0 && !isspace((unsigned char)(*buffer)))
+	buffer++;
+    return buffer;
+}
+
+/*
+ * Skip CONST whitespace
+ */
+PUBLIC CONST char * LYSkipCBlanks ARGS1(
+	CONST char *,	buffer)
+{
+    while (isspace((unsigned char)(*buffer)))
+	buffer++;
+    return buffer;
+}
+
+/*
+ * Skip CONST non-whitespace
+ */
+PUBLIC CONST char * LYSkipCNonBlanks ARGS1(
+	CONST char *,	buffer)
+{
+    while (*buffer != 0 && !isspace((unsigned char)(*buffer)))
+	buffer++;
+    return buffer;
+}
+
+/*
+ * Trim leading blanks from a string
+ */
+PUBLIC void LYTrimLeading ARGS1(
+	char *, 	buffer)
+{
+    char *skipped = LYSkipBlanks(buffer);
+    while ((*buffer++ = *skipped++) != 0)
+	;
+}
+
+/*
+ * Trim trailing blanks from a string
+ */
+PUBLIC void LYTrimTrailing ARGS1(
+	char *, 	buffer)
+{
+    size_t i = strlen(buffer);
+    while (i != 0 && isspace((unsigned char)buffer[i-1]))
+	buffer[--i] = 0;
+}
 
 /*
 **  Display the current value of the string and allow the user
@@ -794,7 +1419,7 @@ PUBLIC int LYEdit1 ARGS4(
     if (MaxLen <= 0)
 	return(0); /* Be defensive */
 
-    length=strlen(&Buf[0]);
+    length = strlen(&Buf[0]);
     StrLen = length;
 
     switch (action) {
@@ -940,13 +1565,11 @@ PUBLIC int LYEdit1 ARGS4(
 	break;
 
     case LYE_UPPER:
-	for (i = 0; Buf[i]; i++)
-	   Buf[i] = TOUPPER(Buf[i]);
+	LYUpperCase(Buf);
 	break;
 
     case LYE_LOWER:
-	for (i = 0; Buf[i]; i++)
-	   Buf[i] = TOLOWER(Buf[i]);
+	LYLowerCase(Buf);
 	break;
 
     default:
@@ -1116,7 +1739,6 @@ again:
 	     */
 	    strcpy(inputline, MyEdit.buffer);
 	    return(ch);
-	    break;
 
 	case LYE_ABORT:
 	    /*
@@ -1124,7 +1746,6 @@ again:
 	     */
 	    inputline[0] = '\0';
 	    return(-1);
-	    break;
 
 	case LYE_LKCMD:
 	    /*
@@ -1140,6 +1761,34 @@ again:
     }
 }
 
+/*
+ *  A replacement for 'strsep()'
+ */
+PUBLIC char *LYstrsep ARGS2(
+	char **,	stringp,
+	CONST char *,	delim)
+{
+    char *tmp, *out;
+
+    if (!stringp || !*stringp)		/* nothing to do? */
+	return 0;			/* then don't fall on our faces */
+
+    if (!**stringp) {			/* empty string: */
+	*stringp = 0;			/* let caller see he's done; */
+	return 0;			/* no tokens in an empty string */
+    }
+
+    out = *stringp;			/* save the start of the string */
+    tmp = strpbrk(*stringp, delim);
+    if (tmp) {
+	*tmp = '\0';			/* terminate the substring with \0 */
+	*stringp = ++tmp;		/* point at the next substring */
+    }
+    else *stringp = 0;			/* this was the last substring: */
+					/* let caller see he's done */
+    return out;
+}
+
 /*
  *  LYstrstr will find the first occurrence of the string
  *  pointed to by tarptr in the string pointed to by chptr.
@@ -1148,7 +1797,7 @@ again:
  */
 PUBLIC char * LYstrstr ARGS2(
 	char *, 	chptr,
-	char *, 	tarptr)
+	CONST char *, 	tarptr)
 {
     int len = strlen(tarptr);
 
@@ -1565,8 +2214,7 @@ PUBLIC char * SNACopy ARGS3(
     if (src) {
 	*dest = (char *)calloc(1, n + 1);
 	if (*dest == NULL) {
-	    if (TRACE)
-		fprintf(stderr, "Tried to calloc %d bytes\n", n);
+	    CTRACE(tfp, "Tried to calloc %d bytes\n", n);
 	    outofmem(__FILE__, "SNACopy");
 	}
 	strncpy (*dest, src, n);
@@ -1627,8 +2275,8 @@ PUBLIC char * SNACat ARGS3(
 **   may be interpreted as equal, but this side effect is negligible
 **   if the user search string is more than one character long.  - LP
 **
-**   Currently we enable new technique only for DOS/WINDOWS display charsets
-**   and also for EXP_8BIT_TOUPPER compilation symbol.
+**   We enable new technique only if  DisplayCharsetMatchLocale = FALSE
+**   (see description in LYCharSets.c)
 */
 PUBLIC int UPPER8 ARGS2(int,ch1, int,ch2)
 {
@@ -1640,16 +2288,9 @@ PUBLIC int UPPER8 ARGS2(int,ch1, int,ch2)
     /* case-insensitive match for upper half */
     if ((unsigned char)ch1 > 127 && (unsigned char)ch2 >127)
     {
-	CONST char *disp_charset;
-	disp_charset = LYCharSet_UC[current_char_set].MIMEname;
-
-#if !defined(EXP_8BIT_TOUPPER)
-	if  (!(strncasecomp(disp_charset, "cp", 2) ||
-		strncasecomp(disp_charset, "windows", 7))) {
-
-	return(TOUPPER(ch1) - TOUPPER(ch2)); /* old-style */
-	} else 
-#endif
+	if (DisplayCharsetMatchLocale)
+	   return(TOUPPER(ch1) - TOUPPER(ch2)); /* old-style */
+	else
 	{
 	/* compare "7bit approximation" for letters >127   */
 	/* BTW, if we remove the check for >127 above	   */
@@ -1658,7 +2299,7 @@ PUBLIC int UPPER8 ARGS2(int,ch1, int,ch2)
 	int charset_in, charset_out, uck1, uck2;
 	char replace_buf1 [10], replace_buf2 [10];
 
-	charset_in  = UCGetLYhndl_byMIME(disp_charset);
+	charset_in  = current_char_set;  /* display character set */
 	charset_out = UCGetLYhndl_byMIME("us-ascii");
 
 	uck1 = UCTransCharStr(replace_buf1, sizeof(replace_buf1), ch1,
@@ -1671,6 +2312,7 @@ PUBLIC int UPPER8 ARGS2(int,ch1, int,ch2)
 
 	/* check to be sure we have not lost any strange characters */
 	/* which are not found in def7_uni.tbl but _equal_ in fact. */
+	/* this also applied for "x-transparent" display mode.	    */
 	if ((unsigned char)ch1==(unsigned char)ch2)
 	    return(0);	 /* match */
 	}
